@@ -8,6 +8,9 @@
 #include <QtQuick3DAssetUtils/private/qssgrtutilities_p.h>
 #include <QtQuick3DAssetImport/private/qssgassetimportmanager_p.h>
 #include <QtQuick3DRuntimeRender/private/qssgrenderbuffermanager_p.h>
+#if QT_CONFIG(mimetype)
+#include <QtCore/qmimedatabase.h>
+#endif
 
 /*!
     \qmltype RuntimeLoader
@@ -19,11 +22,12 @@
     The RuntimeLoader type provides a way to load a 3D asset directly from source at runtime,
     without converting it to QtQuick3D's internal format first.
 
-    Qt 6.2 supports the loading of glTF version 2.0 files in both in text (.gltf) and binary (.glb) formats.
+    RuntimeLoader supports .obj and glTF version 2.0 files in both in text (.gltf) and binary
+    (.glb) formats.
 */
 
 /*!
-    \qmlproperty url QtQuick3D::RuntimeLoader::source
+    \qmlproperty url RuntimeLoader::source
 
     This property holds the location of the source file containing the 3D asset.
     Changing this property will unload the current asset and attempt to load an asset from
@@ -33,7 +37,7 @@
 */
 
 /*!
-    \qmlproperty enumeration QtQuick3D::RuntimeLoader::status
+    \qmlproperty enumeration RuntimeLoader::status
 
     This property holds the status of the latest load operation.
 
@@ -48,7 +52,7 @@
 */
 
 /*!
-    \qmlproperty string QtQuick3D::RuntimeLoader::errorString
+    \qmlproperty string RuntimeLoader::errorString
 
     This property holds a human-readable string indicating the status of the latest load operation.
 
@@ -56,7 +60,7 @@
 */
 
 /*!
-    \qmlproperty Bounds QtQuick3D::RuntimeLoader::bounds
+    \qmlproperty Bounds RuntimeLoader::bounds
 
     This property describes the extents of the bounding volume around the imported model.
 
@@ -66,7 +70,7 @@
 */
 
 /*!
-    \qmlproperty QtQuick3D::Instancing QtQuick3D::RuntimeLoader::instancing
+    \qmlproperty Instancing RuntimeLoader::instancing
 
     If this property is set, the imported model will not be rendered normally. Instead, a number of
     instances will be rendered, as defined by the instance table.
@@ -111,6 +115,48 @@ void QQuick3DRuntimeLoader::componentComplete()
     loadSource();
 }
 
+QStringList QQuick3DRuntimeLoader::supportedExtensions()
+{
+    static QStringList extensions;
+    if (!extensions.isEmpty())
+        return extensions;
+
+    static const QStringList supportedExtensions = { QLatin1StringView("obj"),
+                                                     QLatin1StringView("gltf"),
+                                                     QLatin1StringView("glb")};
+
+    QSSGAssetImportManager importManager;
+    const auto types = importManager.getImporterPluginInfos();
+
+    for (const auto &t : types) {
+        for (const QString &extension : t.inputExtensions) {
+            if (supportedExtensions.contains(extension))
+                extensions << extension;
+        }
+    }
+    return extensions;
+}
+
+#if QT_CONFIG(mimetype)
+QList<QMimeType> QQuick3DRuntimeLoader::supportedMimeTypes()
+{
+    static QList<QMimeType> mimeTypes;
+    if (!mimeTypes.isEmpty())
+        return mimeTypes;
+
+    const QStringList &extensions = supportedExtensions();
+
+    QMimeDatabase db;
+    for (const auto &ext : extensions) {
+        // TODO: Change to db.mimeTypesForExtension(ext), once it is implemented (QTBUG-118566)
+        const QString fileName = QLatin1StringView("test.") + ext;
+        mimeTypes << db.mimeTypesForFileName(fileName);
+    }
+
+    return mimeTypes;
+}
+#endif
+
 static void boxBoundsRecursive(const QQuick3DNode *baseNode, const QQuick3DNode *node, QQuick3DBounds3 &accBounds)
 {
     if (!node)
@@ -118,9 +164,7 @@ static void boxBoundsRecursive(const QQuick3DNode *baseNode, const QQuick3DNode
 
     if (auto *model = qobject_cast<const QQuick3DModel *>(node)) {
         auto b = model->bounds();
-        QSSGBounds2BoxPoints corners;
-        b.bounds.expand(corners);
-        for (const auto &point : corners) {
+        for (const QVector3D point : b.bounds.toQSSGBoxPoints()) {
             auto p = model->mapPositionToNode(const_cast<QQuick3DNode *>(baseNode), point);
             if (Q_UNLIKELY(accBounds.bounds.isEmpty()))
                 accBounds.bounds = { p, p };
@@ -178,24 +222,26 @@ void QQuick3DRuntimeLoader::loadSource()
         break;
     }
 
+    if (m_status == Status::Success) {
+        // We create a dummy root node here, as it will be the parent to the first-level nodes
+        // and resources. If we use 'this' those first-level nodes/resources won't be deleted
+        // when a new scene is loaded.
+        m_root = new QQuick3DNode(this);
+        m_imported = QSSGRuntimeUtils::createScene(*m_root, scene);
+        m_assetId = scene.id;
+        m_boundsDirty = true;
+        m_instancingChanged = m_instancing != nullptr;
+        updateModels();
+        // Cleanup scene before deleting.
+        scene.cleanup();
+    } else {
+        m_source.clear();
+        emit sourceChanged();
+    }
+
     emit statusChanged();
     emit errorStringChanged();
 
-    if (m_status != Status::Success) {
-        m_source.clear();
-        emit sourceChanged();
-        return;
-    }
-
-    // We create a dummy root node here, as it will be the parent to the first-level nodes
-    // and resources. If we use 'this' those first-level nodes/resources won't be deleted
-    // when a new scene is loaded.
-    m_root = new QQuick3DNode(this);
-    m_imported = QSSGRuntimeUtils::createScene(*m_root, scene);
-    m_assetId = scene.id;
-    m_boundsDirty = true;
-    m_instancingChanged = m_instancing != nullptr;
-    updateModels();
 }
 
 void QQuick3DRuntimeLoader::updateModels()
@@ -257,6 +303,10 @@ void QQuick3DRuntimeLoader::setInstancing(QQuick3DInstancing *newInstancing)
 {
     if (m_instancing == newInstancing)
         return;
+
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DRuntimeLoader::setInstancing,
+                                         newInstancing, m_instancing);
+
     m_instancing = newInstancing;
     m_instancingChanged = true;
     updateModels();
