@@ -1,4 +1,4 @@
-// Copyright (C) 2021 The Qt Company Ltd.
+// Copyright (C) 2022 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef QSSGSCENEDESCRIPTION_P_H
@@ -42,6 +42,7 @@
 #include <qmetatype.h>
 #include <QtQuick3DUtils/private/qssginvasivelinkedlist_p.h>
 
+#include <QtGui/qquaternion.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -69,70 +70,70 @@ struct Q_QUICK3DASSETUTILS_EXPORT Scene
 {
     using ResourceNodes = QVarLengthArray<Node *>;
     using MeshStorage = QVector<QSSGMesh::Mesh>;
-    using Allocator = QSSGPerFrameAllocator;
     using Animations = QVector<Animation *>;
 
     // Root node, usually an empty 'transform' node.
     Node *root = nullptr;
     QString id; // Don't make any assumption about the content of this id...
     ResourceNodes resources;
-    Allocator allocator;
     MeshStorage meshStorage;
     Animations animations;
+    QString sourceDir;
     mutable quint16 nodeId = 0;
 
-    template<typename T, typename... Args>
-    Q_REQUIRED_RESULT inline auto create(Args&&... args)
-    {
-        using Tt = rm_cvref_t<T>;
-        return new (allocator.allocate(sizeof(Tt)))Tt(std::forward<Args>(args)...);
-    }
-
     void reset();
+    void cleanup();
 };
 
 struct Q_QUICK3DASSETUTILS_EXPORT PropertyCall
 {
+    virtual ~PropertyCall() = default;
     virtual bool set(QQuick3DObject &, const char *, const void *) = 0;
+    virtual bool set(QQuick3DObject &, const char *, const QVariant &) = 0;
     virtual bool get(const QQuick3DObject &, const void *[]) const = 0;
 };
 
-struct Value
-{
-    QMetaType mt;
-    void *dptr;
-};
+Q_QUICK3DASSETUTILS_EXPORT void destructValue(QVariant &value);
 
 struct Flag
 {
     QMetaEnum me;
-    quintptr value;
+    int value;
 };
 
-struct BufferView {
-    using type = QByteArray;
-    QByteArrayView view;
-};
-struct UrlView : BufferView { using type = QUrl; };
-struct StringView : BufferView { using type = QString; };
-
 struct Property
 {
+    ~Property();
     enum class Type { Static, Dynamic };
-    Value value;
-    const char *name = nullptr;
+    QVariant value;
+    QByteArray name;
     QSSGSceneDesc::PropertyCall *call = nullptr;
-    Property *next = nullptr;
     Type type = Type::Static;
 };
 
+inline Property::~Property()
+{
+    delete call;
+    destructValue(value);
+}
+
+Q_QUICK3DASSETUTILS_EXPORT void destructNode(QSSGSceneDesc::Node &node);
+
 struct NodeList
 {
+    NodeList(void * const *data, qsizetype n)
+    {
+        const auto size = sizeof(Node *) * n;
+        head = reinterpret_cast<Node **>(malloc(size));
+        memcpy(head, data, size);
+        count = n;
+    }
+    ~NodeList() { if (head) free(head); }
     Node **head = nullptr;
     qsizetype count = -1;
 };
 
-struct Node
+struct Q_QUICK3DASSETUTILS_EXPORT Node
 {
     // Node type
     enum class Type : quint8
@@ -154,19 +155,21 @@ struct Node
     // Runtime type type mapping between this type and the QtQuick3D type
     using RuntimeType = QSSGRenderGraphObject::Type;
 
-    explicit Node(QByteArrayView name, Node::Type type, Node::RuntimeType rt)
+    explicit Node(QByteArray name, Node::Type type, Node::RuntimeType rt)
         : name(name)
         , runtimeType(rt)
         , nodeType(type) {}
     explicit Node(Node::Type type, Node::RuntimeType rt)
         : Node(nullptr, type, rt) {}
 
-    QByteArrayView name;
+    virtual ~Node();
+    void cleanupChildren();
+
+    QByteArray name;
     Scene *scene = nullptr;
     QObject *obj = nullptr;
-    Node *next = nullptr;
-    using ChildList = QSSGInvasiveSingleLinkedList<Node, &Node::next>;
-    using PropertyList = QSSGInvasiveSingleLinkedList<Property, &Property::next>;
+    using ChildList = QList<Node *>;
+    using PropertyList = QList<Property *>;
     ChildList children;
     PropertyList properties;
     quint16 id = 0;
@@ -178,11 +181,9 @@ template<typename T>
 static constexpr bool is_node_v = std::is_base_of_v<Node, T>;
 
 // Set up type mapping from a QQuick3D type to a SceneDesc type
-// and verfiy that the node is trivially destructable.
 template <typename T> struct TypeMap {};
 #define QSSG_DECLARE_NODE(NODE) \
 static_assert(is_node_v<NODE>, #NODE " - does not inherit from Node!"); \
-static_assert (std::is_trivially_destructible_v<NODE>, #NODE " - needs to be trivially destructable!"); \
 template <> struct TypeMap<NODE::type> { using type = QSSGSceneDesc::NODE; };
 
 template<typename T>
@@ -192,14 +193,14 @@ using as_node_type_t = typename TypeMap<T>::type;
 
 QSSG_DECLARE_NODE(Node)
 
-struct Texture : Node
+struct Q_QUICK3DASSETUTILS_EXPORT Texture : Node
 {
     using type = QQuick3DTexture;
-    Texture(Node::RuntimeType rt) : Node(Node::Type::Texture, rt) {}
+    explicit Texture(Node::RuntimeType rt, const QByteArray &name = {});
 };
 QSSG_DECLARE_NODE(Texture)
 
-struct TextureData : Node
+struct Q_QUICK3DASSETUTILS_EXPORT TextureData : Node
 {
     using type = QQuick3DTextureData;
     enum class Flags : quint8
@@ -207,71 +208,61 @@ struct TextureData : Node
         Compressed = 0x1
     };
 
-    using Format = QQuick3DTextureData::Format;
-    explicit TextureData(QByteArrayView dataref, QSize size, Format format, quint8 flags = 0, QByteArrayView name = QByteArrayView())
-        : Node(name, Node::Type::Texture, RuntimeType::TextureData)
-        , data(dataref)
-        , sz(size)
-        , fmt(format)
-        , flgs(flags)
-    {}
-    QByteArrayView data;
+    explicit TextureData(const QByteArray &textureData, QSize size, const QByteArray &format, quint8 flags = 0, QByteArray name = {});
+    QByteArray data;
     QSize sz;
-    Format fmt;
+    QByteArray fmt;
     quint8 flgs;
 };
 QSSG_DECLARE_NODE(TextureData)
 
-struct Material : Node
+struct Q_QUICK3DASSETUTILS_EXPORT Material : Node
 {
     using type = QQuick3DMaterial;
-    explicit Material(Node::RuntimeType rt) : Node(Node::Type::Material, rt) {}
+    explicit Material(Node::RuntimeType rt);
 };
 QSSG_DECLARE_NODE(Material)
 
 // The mesh is a special node, as it's not really a node type but
 // a handle to a mesh that will be turned into a source URL...
-struct Mesh : Node
+struct Q_QUICK3DASSETUTILS_EXPORT Mesh : Node
 {
-    explicit Mesh(QByteArrayView name, qsizetype index)
-        : Node(name, Node::Type::Mesh, RuntimeType::Node)
-        , idx(index)
-    {}
+    explicit Mesh(QByteArray name, qsizetype index);
     qsizetype idx; // idx to the mesh data in the mesh data storage (see Scene).
 };
 
-struct Model : Node
+struct Q_QUICK3DASSETUTILS_EXPORT Model : Node
 {
     using type = QQuick3DModel;
-    Model() : Node(Node::Type::Model, Node::RuntimeType::Model) {}
+    Model();
 };
 QSSG_DECLARE_NODE(Model)
 
-struct Camera : Node
+struct Q_QUICK3DASSETUTILS_EXPORT Camera : Node
 {
     using type = QQuick3DCamera;
-    explicit Camera(RuntimeType rt) : Node(Node::Type::Camera, rt) {}
+    explicit Camera(RuntimeType rt);
 };
 QSSG_DECLARE_NODE(Camera)
 
-struct Light : Node
+struct Q_QUICK3DASSETUTILS_EXPORT Light : Node
 {
     using type = QQuick3DAbstractLight;
-    explicit Light(RuntimeType rt) : Node(Node::Type::Light, rt) {}
+    explicit Light(RuntimeType rt);
 };
 QSSG_DECLARE_NODE(Light)
 
-struct Skin : Node
+struct Q_QUICK3DASSETUTILS_EXPORT Skin : Node
 {
     using type = QQuick3DSkin;
-    Skin() : Node(Node::Type::Skin, Node::RuntimeType::Skin) {}
+    Skin();
 };
 QSSG_DECLARE_NODE(Skin)
 
-struct Skeleton : Node
+struct Q_QUICK3DASSETUTILS_EXPORT Skeleton : Node
 {
     using type = QQuick3DSkeleton;
-    Skeleton() : Node(Node::Type::Skeleton, Node::RuntimeType::Skeleton) {}
+    Skeleton();
     // Skeleton is a virtual node, which is added for the start of the joint heirarchy.
     // parent - joint 1     ->      parent - skeleton - joint 1
     //        - joint 2                               - joint 2
@@ -281,28 +272,31 @@ struct Skeleton : Node
 };
 QSSG_DECLARE_NODE(Skeleton)
 
-struct Joint : Node
+struct Q_QUICK3DASSETUTILS_EXPORT Joint : Node
 {
     using type = QQuick3DJoint;
-    Joint() : Node(Node::Type::Joint, Node::RuntimeType::Joint) {}
+    Joint();
 };
 QSSG_DECLARE_NODE(Joint)
 
-struct MorphTarget : Node
+struct Q_QUICK3DASSETUTILS_EXPORT MorphTarget : Node
 {
     using type = QQuick3DMorphTarget;
-    MorphTarget() : Node(Node::Type::MorphTarget, Node::RuntimeType::MorphTarget) {}
+    MorphTarget();
 };
 QSSG_DECLARE_NODE(MorphTarget)
 
+// We keep our own list data structure, since Qt does not have a variant list where all the
+// elements have the same type, and using a list of QVariant is very inefficient
 struct ListView
 {
-    Value head;
+    ~ListView() { if (data) free(data); }
+    QMetaType mt;
+    void *data = nullptr;
     qsizetype count = -1;
 };
 
 Q_QUICK3DASSETUTILS_EXPORT QMetaType listViewMetaType();
-Q_QUICK3DASSETUTILS_EXPORT QMetaType flagMetaType();
 
 struct Animation
 {
@@ -358,9 +352,8 @@ struct Animation
         QVector4D value;
         float time = 0.0f;
         quint16 flag = 0;
-        KeyPosition *next = nullptr;
     };
-    using Keys = QSSGInvasiveSingleLinkedList<Animation::KeyPosition, &Animation::KeyPosition::next>;
+    using Keys = QList<Animation::KeyPosition *>;
 
     struct Channel
     {
@@ -381,17 +374,20 @@ struct Animation
 
         Node *target = nullptr;
         Keys keys;
-        Channel *next = nullptr;
         TargetType targetType = TargetType::Property;
         TargetProperty targetProperty = TargetProperty::Unknown;
     };
-    using Channels = QSSGInvasiveSingleLinkedList<Animation::Channel, &Animation::Channel::next>;
+    using Channels = QList<Animation::Channel *>;
 
     Channels channels;
     // It stores the length of this Animation, every keys in every channels in
     // an animation will have the same KeyType and it will be a type of
     // the length
     float length = 0.0f;
+
+    float framesPerSecond = 0.0f; // for translation back to frames
+
+    QByteArray name;
 };
 
 // Add a child node to parent node.
@@ -457,15 +453,16 @@ struct PropertyProxySetter : PropertyCall
     bool get(const QQuick3DObject &, const void *[]) const override { return false; }
     bool set(QQuick3DObject &that, const char *name, const void *value) override
     {
-        if (value) {
-            if constexpr (std::is_pointer_v<typename FuncType<Setter>::Arg2>)
-                call(that, name, reinterpret_cast<typename FuncType<Setter>::Arg2>(const_cast<void *>(value)));
-            else
-                call(that, name, *reinterpret_cast<typename FuncType<Setter>::Arg2Base *>(const_cast<void *>(value)));
-            return true;
-        }
-
-        return false;
+        if constexpr (std::is_pointer_v<typename FuncType<Setter>::Arg2>)
+            call(that, name, reinterpret_cast<typename FuncType<Setter>::Arg2>(const_cast<void *>(value)));
+        else
+            call(that, name, *reinterpret_cast<typename FuncType<Setter>::Arg2Base *>(const_cast<void *>(value)));
+        return true;
+    }
+    bool set(QQuick3DObject &that, const char *name, const QVariant &var) override
+    {
+        call(that, name, qvariant_cast<typename FuncType<Setter>::Arg2Base>(var));
+        return true;
     }
 };
 
@@ -478,15 +475,17 @@ struct PropertySetter : PropertyCall
     bool get(const QQuick3DObject &, const void *[]) const override { return false; }
     bool set(QQuick3DObject &that, const char *, const void *value) override
     {
-        if (value) {
-            if constexpr (std::is_pointer_v<typename FuncType<Setter>::Arg0>)
-                (qobject_cast<Class *>(&that)->*call)(reinterpret_cast<typename FuncType<Setter>::Arg0>(const_cast<void *>(value)));
-            else
-                (qobject_cast<Class *>(&that)->*call)(*reinterpret_cast<typename FuncType<Setter>::Arg0Base *>(const_cast<void *>(value)));
-            return true;
+        if constexpr (std::is_pointer_v<typename FuncType<Setter>::Arg0>)
+            (qobject_cast<Class *>(&that)->*call)(reinterpret_cast<typename FuncType<Setter>::Arg0>(const_cast<void *>(value)));
+        else {
+            (qobject_cast<Class *>(&that)->*call)(*reinterpret_cast<typename FuncType<Setter>::Arg0Base *>(const_cast<void *>(value)));
         }
-
-        return false;
+        return true;
+    }
+    bool set(QQuick3DObject &that, const char *, const QVariant &var) override
+    {
+        (qobject_cast<Class *>(&that)->*call)(qvariant_cast<typename FuncType<Setter>::Arg0Base>(var));
+        return true;
     }
 };
 
@@ -502,14 +501,31 @@ struct PropertyListSetter : PropertyCall
     bool set(QQuick3DObject &that, const char *, const void *value) override
     {
         if (const auto listView = reinterpret_cast<const ListView *>(value)) {
-            const auto begin = reinterpret_cast<It *>(listView->head.dptr);
-            const auto end = reinterpret_cast<It *>(listView->head.dptr) + listView->count;
-            (qobject_cast<Class *>(&that)->*call)(ListT{begin, end});
+            if (listView->count > 0) {
+                const auto begin = reinterpret_cast<It *>(listView->data);
+                const auto end = reinterpret_cast<It *>(listView->data) + listView->count;
+                (qobject_cast<Class *>(&that)->*call)(ListT{begin, end});
+            } else {
+                (qobject_cast<Class *>(&that)->*call)(ListT{});
+            }
             return true;
         }
 
         return false;
     }
+    bool set(QQuick3DObject &that, const char *, const QVariant &var) override
+    {
+        if (const auto listView = qvariant_cast<const ListView *>(var)) {
+            if (listView->count > 0) {
+                const auto begin = reinterpret_cast<It *>(listView->data);
+                const auto end = reinterpret_cast<It *>(listView->data) + listView->count;
+                (qobject_cast<Class *>(&that)->*call)(ListT{begin, end});
+            } else {
+                (qobject_cast<Class *>(&that)->*call)(ListT{});
+            }
+        }
+        return false;
+    }
 };
 
 template <typename Class, typename T, template <typename> typename List>
@@ -523,44 +539,33 @@ struct PropertyList : PropertyCall
     static_assert(std::is_same_v<ListType, QQmlListProperty<T>>, "Expected QQmlListProperty!");
 
     bool get(const QQuick3DObject &, const void *[]) const override { return false; }
+
+
+    void doSet(QQuick3DObject &that, const QSSGSceneDesc::NodeList &nodeList)
+    {
+        ListType list = (qobject_cast<Class *>(&that)->*listfn)();
+        auto head = reinterpret_cast<as_node_type_t<T> **>(nodeList.head);
+        for (int i = 0, end = nodeList.count; i != end; ++i)
+            list.append(&list, qobject_cast<T *>((*(head + i))->obj));
+    }
+
     bool set(QQuick3DObject &that, const char *, const void *value) override
     {
         if (value) {
-            ListType list = (qobject_cast<Class *>(&that)->*listfn)();
             const auto &nodeList = *reinterpret_cast<const QSSGSceneDesc::NodeList *>(value);
-            auto head = reinterpret_cast<as_node_type_t<T> **>(nodeList.head);
-            for (int i = 0, end = nodeList.count; i != end; ++i)
-                list.append(&list, qobject_cast<T *>((*(head + i))->obj));
+            doSet(that, nodeList);
             return true;
         }
-
         return false;
     }
-};
 
-template <typename T, typename Class>
-struct PropertyGetter : PropertyCall
-{
-    using Getter = T (Class::*)() const;
-    constexpr explicit PropertyGetter(Getter fn) : call(fn) {}
-    Getter call = nullptr;
-    bool get(const QQuick3DObject &, const void *[]) const override { return false; }
-    bool set(QQuick3DObject &, const char *, const void *) override { return false; }
-};
-
-template <typename T, typename Class>
-struct PropertyMember : PropertyCall
-{
-    using Getter = T (Class::*);
-    constexpr explicit PropertyMember(Getter fn) : call(fn) {}
-    Getter call = nullptr;
-    bool get(const QQuick3DObject &, const void *[]) const override { return false; }
-    bool set(QQuick3DObject &that, const char *, const void *value) override {
-        if (value) {
-            (qobject_cast<Class *>(&that)->*call) = (*reinterpret_cast<const T *>(value));
+    bool set(QQuick3DObject &that, const char *, const QVariant &var) override
+    {
+        const auto *nodeList = qvariant_cast<const QSSGSceneDesc::NodeList *>(var);
+        if (nodeList) {
+            doSet(that, *nodeList);
             return true;
         }
-
         return false;
     }
 };
@@ -581,27 +586,25 @@ template<typename Setter, typename T, if_compatible_t<Setter, T> = false>
 static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter setter, T &&value)
 {
      Q_ASSERT(node.scene);
-    static_assert(std::is_trivially_destructible_v<rm_cvref_t<T>>, "Value needs to be trivially destructible!");
-    auto prop = node.scene->create<Property>();
+    auto prop = new Property;
     prop->name = name;
-    prop->call = node.scene->create<decltype(PropertySetter(setter))>(setter);
-    prop->value.mt = QMetaType::fromType<rm_cvref_t<T>>();
-    prop->value.dptr = node.scene->create<T>(std::forward<T>(value));
-    node.properties.push_back(*prop);
+    prop->call = new PropertySetter(setter);
+    prop->value = QVariant::fromValue(std::forward<T>(value));
+    node.properties.push_back(prop);
 }
 
 template<typename Setter, typename T, if_compatible_t<Setter, QFlags<T>> = false>
 static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter setter, QFlags<T> value)
 {
     Q_ASSERT(node.scene);
-    auto prop = node.scene->create<Property>();
+    auto prop = new Property;
     prop->name = name;
-    prop->call = node.scene->create<decltype(PropertySetter(setter))>(setter);
-    prop->value.mt = flagMetaType();
-    prop->value.dptr = node.scene->create<Flag>(Flag{ QMetaEnum::fromType<rm_cvref_t<T>>(), quintptr(value) });
-    node.properties.push_back(*prop);
+    prop->call = new PropertySetter(setter);
+    prop->value = QVariant::fromValue(Flag{ QMetaEnum::fromType<rm_cvref_t<T>>(), value.toInt() });
+    node.properties.push_back(prop);
 }
 
+
 template<typename Setter, typename T, if_compatible_t<Setter, QList<T>> = false>
 static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter setter, QList<T> value)
 {
@@ -609,31 +612,35 @@ static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter sett
     static_assert(!std::is_pointer_v<T>, "Type cannot be a pointer!");
     static_assert(std::is_trivially_destructible_v<T> && std::is_trivially_copy_constructible_v<T>,
             "List parameter type needs to be trivially constructable and trivially destructible!");
-    if (const auto count = value.size()) {
-        const auto asize = count * sizeof(T);
-        auto data = node.scene->allocator.allocate(asize);
-        memcpy(data, value.constData(), asize);
 
-        auto prop = node.scene->create<Property>();
-        prop->name = name;
-        prop->call = node.scene->create<decltype(PropertyListSetter(setter))>(setter);
-        prop->value.mt = listViewMetaType();
-        prop->value.dptr = node.scene->create<ListView>(ListView{ { QMetaType::fromType<rm_cvref_t<T>>(), data }, count });
-        node.properties.push_back(*prop);
+    const auto count = value.size();
+    void *data = nullptr;
+    if (count) {
+        const auto asize = count * sizeof(T);
+        data = malloc(asize); // is freed in ~ListView
+        memcpy(data, value.constData(), asize);
     }
+    auto prop = new Property;
+    prop->name = name;
+    prop->call = new PropertyListSetter(setter);
+    prop->value = QVariant::fromValue(new ListView{ QMetaType::fromType<rm_cvref_t<T>>(), data, count });
+    node.properties.push_back(prop);
 }
 
+
+Q_QUICK3DASSETUTILS_EXPORT QSSGSceneDesc::Property *setProperty(QSSGSceneDesc::Node &node, const char *name, QVariant &&value);
+
 // Calling this will omit any type checking, so make sure the type is handled correctly
 // when it gets used later!
 template<typename Setter>
-static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter setter, QSSGSceneDesc::Value &&value)
+static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter setter, QVariant &&value)
 {
     Q_ASSERT(node.scene);
-    Property *prop = node.scene->create<Property>();
+    Property *prop = new Property;
     prop->name = name;
-    prop->call = node.scene->create<decltype(PropertySetter(setter))>(setter);
+    prop->call = new PropertySetter(setter);
     prop->value = value;
-    node.properties.push_back(*prop);
+    node.properties.push_back(prop);
 }
 
 template<typename Setter, typename Value, if_compatible_proxy_t<Setter, Value> = true>
@@ -641,16 +648,12 @@ static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter sett
 {
     Q_ASSERT(node.scene);
     static_assert(std::is_trivially_destructible_v<rm_cvref_t<Value>>, "Value needs to be trivially destructible!");
-    Property *prop = node.scene->create<Property>();
+    Property *prop = new Property;
     prop->name = name;
-    prop->call = node.scene->create<decltype(PropertyProxySetter(setter))>(setter);
-    prop->value.mt = QMetaType::fromType<rm_cvref_t<Value>>();
-    if constexpr (std::is_pointer_v<rm_cvref_t<Value>>)
-        prop->value.dptr = value;
-    else
-       prop->value.dptr = node.scene->create<Value>(std::forward<Value>(value));
+    prop->call = new PropertyProxySetter(setter);
+    prop->value = QVariant::fromValue(value);
     prop->type = type;
-    node.properties.push_back(*prop);
+    node.properties.push_back(prop);
 }
 
 template<typename Setter, typename ViewValue, if_compatible_t<Setter, typename ViewValue::type> = false>
@@ -658,25 +661,23 @@ static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter sett
 {
     Q_ASSERT(node.scene);
     static_assert(std::is_same_v<typename ViewValue::type, typename FuncType<Setter>::Arg0Base>, "Type cannot be mapped to slot argument");
-    Property *prop = node.scene->create<Property>();
+    Property *prop = new Property;
     prop->name = name;
-    prop->call = node.scene->create<decltype(PropertySetter(setter))>(setter);
-    prop->value.mt = QMetaType::fromType<rm_cvref_t<ViewValue>>();
-    prop->value.dptr = node.scene->create<ViewValue>(std::move(view));
-    node.properties.push_back(*prop);
+    prop->call = new PropertySetter(setter);
+    prop->value = QVariant::fromValue(view);
+    node.properties.push_back(prop);
 }
 
 template<typename Setter, typename Value, if_compatible_t<Setter, as_scene_type_t<Value> *> = true>
 static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter setter, Value *value)
 {
     Q_ASSERT(node.scene);
-    auto &scene = node.scene;
-    Property *prop = scene->create<Property>();
+    Property *prop = new Property;
     prop->name = name;
-    prop->call = scene->create<decltype(PropertySetter(setter))>(setter);
-    prop->value.mt = QMetaType::fromType<Node *>(); // Always 'Node', the Node class itself contains more fine grain type information.
-    prop->value.dptr = value;
-    node.properties.push_back(*prop);
+    prop->call = new PropertySetter(setter);
+    // Always 'Node', the Node class itself contains more fine grain type information.
+    prop->value = QVariant::fromValue(static_cast<Node *>(value));
+    node.properties.push_back(prop);
 }
 
 // Overloaded function for setting a type to a property that's a QQmlListProperty.
@@ -685,21 +686,13 @@ static void setProperty(QSSGSceneDesc::Node &node, const char *name, Setter sett
 {
     Q_ASSERT(node.scene);
     if (!list.isEmpty()) {
-        auto &scene = node.scene;
-        NodeList *l = scene->create<NodeList>();
-        {
-            const auto size = sizeof(Node *) * list.size();
-            l->head = reinterpret_cast<Node **>(scene->allocator.allocate(size));
-            memcpy(l->head, list.data(), size);
-            l->count = list.size();
-        }
+        NodeList *l = new NodeList(reinterpret_cast<void * const*>(list.constData()), list.count());
 
-        Property *prop = scene->create<Property>();
+        Property *prop = new Property;
         prop->name = name;
-        prop->call = scene->create<decltype(PropertyList(setter))>(setter);
-        prop->value.mt = QMetaType::fromType<QSSGSceneDesc::NodeList *>();
-        prop->value.dptr = l;
-        node.properties.push_back(*prop);
+        prop->call = new PropertyList(setter);
+        prop->value = QVariant::fromValue(l);
+        node.properties.push_back(prop);
     }
 }
 
@@ -721,9 +714,6 @@ Q_DECLARE_METATYPE(QSSGSceneDesc::MorphTarget)
 Q_DECLARE_METATYPE(QSSGSceneDesc::NodeList)
 Q_DECLARE_METATYPE(QSSGSceneDesc::Animation)
 
-Q_DECLARE_METATYPE(QSSGSceneDesc::BufferView)
-Q_DECLARE_METATYPE(QSSGSceneDesc::UrlView)
-Q_DECLARE_METATYPE(QSSGSceneDesc::StringView)
 Q_DECLARE_METATYPE(QSSGSceneDesc::ListView)
 
 Q_DECLARE_METATYPE(QSSGSceneDesc::Flag)
