@@ -91,7 +91,7 @@ QT_BEGIN_NAMESPACE
 
     Range: \c{[0.0, 1.0]}
 
-    Default value: \c{0}
+    Default value: \c{0.1}
 
     \sa csmSplit2, csmSplit3
     \note This property is only used when DirectionalLight::csmNumSplits is greater than \c{0}.
@@ -154,6 +154,17 @@ QT_BEGIN_NAMESPACE
     Default value: \c{0.05}
 */
 
+/*!
+    \qmlproperty bool DirectionalLight::lockShadowmapTexels
+    \since 6.9
+
+    When this property is enabled, the shadowmap texels are "locked" in position in the
+    world eliminating shadow edge shimmering at the cost of bigger shadowmap texels.
+    It works by using uniform sized, and texel aligned shadowmaps for each cascade.
+
+    Default value: \c{false}
+*/
+
 QQuick3DDirectionalLight::QQuick3DDirectionalLight(QQuick3DNode *parent)
     : QQuick3DAbstractLight(*(new QQuick3DNodePrivate(QQuick3DNodePrivate::Type::DirectionalLight)), parent) {}
 
@@ -182,6 +193,11 @@ float QQuick3DDirectionalLight::csmBlendRatio() const
     return m_csmBlendRatio;
 }
 
+bool QQuick3DDirectionalLight::lockShadowmapTexels() const
+{
+    return m_lockShadowmapTexels;
+}
+
 void QQuick3DDirectionalLight::setCsmSplit1(float newcsmSplit1)
 {
     newcsmSplit1 = qBound(0.0f, newcsmSplit1, 1.0f);
@@ -242,6 +258,17 @@ void QQuick3DDirectionalLight::setCsmBlendRatio(float newcsmBlendRatio)
     update();
 }
 
+void QQuick3DDirectionalLight::setLockShadowmapTexels(bool newLockShadowmapTexels)
+{
+    if (m_lockShadowmapTexels == newLockShadowmapTexels)
+        return;
+
+    m_lockShadowmapTexels = newLockShadowmapTexels;
+    emit lockShadowmapTexelsChanged();
+    m_dirtyFlags.setFlag(QQuick3DAbstractLight::DirtyFlag::ShadowDirty);
+    update();
+}
+
 QSSGRenderGraphObject *QQuick3DDirectionalLight::updateSpatialNode(QSSGRenderGraphObject *node)
 {
     if (!node) {
@@ -256,6 +283,7 @@ QSSGRenderGraphObject *QQuick3DDirectionalLight::updateSpatialNode(QSSGRenderGra
         light->m_csmSplit3 = m_csmSplit3;
         light->m_csmNumSplits = m_csmNumSplits;
         light->m_csmBlendRatio = m_csmBlendRatio;
+        light->m_lockShadowmapTexels = m_lockShadowmapTexels;
     }
 
     QQuick3DAbstractLight::updateSpatialNode(node); // Marks the light node dirty if m_dirtyFlags != 0
