@@ -13,7 +13,7 @@ QT_BEGIN_NAMESPACE
     \qmltype Instancing
     \inherits Object3D
     \inqmlmodule QtQuick3D
-    \instantiates QQuick3DInstancing
+    \nativetype QQuick3DInstancing
     \since 6.2
     \brief Base type for instance tables.
 
@@ -95,6 +95,34 @@ QT_BEGIN_NAMESPACE
     \note The sorting increases the frame preparation time especially with large instance counts.
 */
 
+/*!
+    \qmlproperty vector3d Instancing::shadowBoundsMinimum
+
+    Sets the minimum bounds used when calculating the shadow map bounds of the models in the
+    instance table.
+
+    Default value: \c{(1, 1, 1)}
+
+    \note This property is only enabled when the respective components of
+    Instancing::shadowBoundsMinimum are smaller than those in
+    Instancing::shadowBoundsMaximum. Otherwise the bounds are calculated automatically.
+    \sa shadowBoundsMaximum
+*/
+
+/*!
+    \qmlproperty vector3d Instancing::shadowBoundsMaximum
+
+    Sets the maximum bounds used when calculating the shadow map bounds of the models in the
+    instance table.
+
+    Default value: \c{(-1, -1, -1)}
+
+    \note This property is only enabled when the respective components of
+    Instancing::shadowBoundsMinimum are smaller than those in
+    Instancing::shadowBoundsMaximum. Otherwise the bounds are calculated automatically.
+    \sa shadowBoundsMinimum
+*/
+
 /*!
     \class QQuick3DInstancing
     \inmodule QtQuick3D
@@ -161,6 +189,18 @@ bool QQuick3DInstancing::depthSortingEnabled() const
     return d->m_depthSortingEnabled;
 }
 
+QVector3D QQuick3DInstancing::shadowBoundsMinimum() const
+{
+    Q_D(const QQuick3DInstancing);
+    return d->m_shadowBoundsMinimum;
+}
+
+QVector3D QQuick3DInstancing::shadowBoundsMaximum() const
+{
+    Q_D(const QQuick3DInstancing);
+    return d->m_shadowBoundsMaximum;
+}
+
 const QQuick3DInstancing::InstanceTableEntry *QQuick3DInstancing::getInstanceEntry(int index)
 {
     const QByteArray data = getInstanceBuffer(nullptr);
@@ -325,6 +365,28 @@ void QQuick3DInstancing::setDepthSortingEnabled(bool enabled)
     emit depthSortingEnabledChanged();
 }
 
+void QQuick3DInstancing::setShadowBoundsMinimum(const QVector3D &newShadowBoundsMinimum)
+{
+    Q_D(QQuick3DInstancing);
+    if (d->m_shadowBoundsMinimum == newShadowBoundsMinimum)
+        return;
+
+    d->m_shadowBoundsMinimum = newShadowBoundsMinimum;
+    d->dirty(QQuick3DObjectPrivate::DirtyType::Content);
+    emit shadowBoundsMinimumChanged();
+}
+
+void QQuick3DInstancing::setShadowBoundsMaximum(const QVector3D &newShadowBoundsMaximum)
+{
+    Q_D(QQuick3DInstancing);
+    if (d->m_shadowBoundsMinimum == newShadowBoundsMaximum)
+        return;
+
+    d->m_shadowBoundsMaximum = newShadowBoundsMaximum;
+    d->dirty(QQuick3DObjectPrivate::DirtyType::Content);
+    emit shadowBoundsMaximumChanged();
+}
+
 /*!
   Mark that the instance data has changed and must be uploaded again.
 
@@ -348,7 +410,7 @@ QSSGRenderGraphObject *QQuick3DInstancing::updateSpatialNode(QSSGRenderGraphObje
         emit instanceNodeDirty();
         d->m_instanceDataChanged = true;
     }
-
+    QQuick3DObject::updateSpatialNode(node);
     auto effectiveInstanceCount = [d]() {
         if (d->m_instanceCountOverride >= 0)
             return qMin(d->m_instanceCount, d->m_instanceCountOverride);
@@ -365,6 +427,8 @@ QSSGRenderGraphObject *QQuick3DInstancing::updateSpatialNode(QSSGRenderGraphObje
     d->m_instanceCountOverrideChanged = false;
     instanceTable->setHasTransparency(d->m_hasTransparency);
     instanceTable->setDepthSorting(d->m_depthSortingEnabled);
+    instanceTable->setShadowBoundsMinimum(d->m_shadowBoundsMinimum);
+    instanceTable->setShadowBoundsMaximum(d->m_shadowBoundsMaximum);
     return node;
 }
 
@@ -385,7 +449,7 @@ static inline QQuick3DInstancing::InstanceTableEntry calculate(const QVector3D &
     xform(1, 3) += position[1];
     xform(2, 3) += position[2];
 
-    auto linearColor = color::sRGBToLinear(color);
+    auto linearColor = QSSGUtils::color::sRGBToLinear(color);
 
     return {
         xform.row(0),
@@ -451,7 +515,7 @@ QQuick3DInstancing::InstanceTableEntry QQuick3DInstancing::calculateTableEntryFr
     xform(1, 3) += position[1];
     xform(2, 3) += position[2];
 
-    auto linearColor = color::sRGBToLinear(color);
+    auto linearColor = QSSGUtils::color::sRGBToLinear(color);
 
     return {
         xform.row(0),
@@ -489,6 +553,24 @@ QQuick3DInstancing::InstanceTableEntry QQuick3DInstancing::calculateTableEntryFr
     }
     \endqml
 
+    It is also possible to populate the instances property by just adding children to the
+    InstanceList. The following example is equivalent to the previous one:
+    \qml
+    InstanceList {
+        id: manualInstancing
+        InstanceListEntry {
+            position: Qt.vector3d(0, 0, -60)
+            eulerRotation: Qt.vector3d(-10, 0, 30)
+            color: "red"
+        }
+        InstanceListEntry {
+            position: Qt.vector3d(50, 10, 100)
+            eulerRotation: Qt.vector3d(0, 180, 0)
+            color: "green"
+        }
+    }
+    \endqml
+
     Each InstanceListEntry is an object that can have property bindings and animations. This gives
     great flexibility, but also causes memory overhead. Therefore, it is not recommended to use
     InstanceList for procedurally generated tables containing thousands (or millions) of
@@ -500,6 +582,7 @@ QQuick3DInstancing::InstanceTableEntry QQuick3DInstancing::calculateTableEntryFr
 
 /*!
     \qmlproperty List<QtQuick3D::InstanceListEntry> InstanceList::instances
+    \qmldefault
 
     This property contains the list of instance definitions. Modifying this list, or any of its elements, will cause the instance table to be updated.
 */
@@ -664,7 +747,7 @@ void QQuick3DInstanceListEntry::setScale(QVector3D scale)
 */
 void QQuick3DInstanceListEntry::setEulerRotation(QVector3D eulerRotation)
 {
-    if (m_eulerRotation == eulerRotation)
+    if (m_useEulerRotation && m_eulerRotation == eulerRotation)
         return;
     m_eulerRotation = eulerRotation;
     m_useEulerRotation = true;
@@ -679,7 +762,7 @@ void QQuick3DInstanceListEntry::setEulerRotation(QVector3D eulerRotation)
 */
 void QQuick3DInstanceListEntry::setRotation(QQuaternion rotation)
 {
-    if (m_rotation == rotation)
+    if (!m_useEulerRotation && m_rotation == rotation)
         return;
 
     m_rotation = rotation;
@@ -689,7 +772,7 @@ void QQuick3DInstanceListEntry::setRotation(QQuaternion rotation)
 }
 
 /*!
-    \qmlproperty vector3d QtQuick3D::InstanceListEntry::color
+    \qmlproperty color QtQuick3D::InstanceListEntry::color
 
     This property specifies the color for the instance.
 */
