@@ -57,7 +57,11 @@ Q_TRACE_POINT(qtquick3d, QSSG_synchronize_exit);
 Q_TRACE_POINT(qtquick3d, QSSG_renderPass_entry, const QString &renderPass);
 Q_TRACE_POINT(qtquick3d, QSSG_renderPass_exit);
 
-static bool dumpRenderTimes = false;
+static bool dumpRenderTimes()
+{
+    static bool val = (qEnvironmentVariableIntValue("QT_QUICK3D_DUMP_RENDERTIMES") > 0);
+    return val;
+}
 
 #if QT_CONFIG(qml_debug)
 
@@ -167,12 +171,12 @@ void SGFramebufferObjectNode::render()
         emit textureChanged();
 
         if (renderer->renderStats())
-            renderer->renderStats()->endRender(dumpRenderTimes);
+            renderer->renderStats()->endRender(dumpRenderTimes());
 
-        if (renderer->requestedFramesCount > 0) {
+        if (renderer->m_requestedFramesCount > 0) {
             scheduleRender();
             requestFullUpdate(window);
-            renderer->requestedFramesCount--;
+            renderer->m_requestedFramesCount--;
         }
     }
 }
@@ -189,7 +193,6 @@ void SGFramebufferObjectNode::handleScreenChange()
 QQuick3DSceneRenderer::QQuick3DSceneRenderer(const std::shared_ptr<QSSGRenderContextInterface> &rci)
     : m_sgContext(rci)
 {
-    dumpRenderTimes = (qEnvironmentVariableIntValue("QT_QUICK3D_DUMP_RENDERTIMES") > 0);
 }
 
 QQuick3DSceneRenderer::~QQuick3DSceneRenderer()
@@ -223,8 +226,11 @@ void QQuick3DSceneRenderer::releaseAaDependentRhiResources()
     delete m_multiViewDepthStencilBuffer;
     m_multiViewDepthStencilBuffer = nullptr;
 
-    delete m_msaaRenderBuffer;
-    m_msaaRenderBuffer = nullptr;
+    delete m_msaaRenderBufferLegacy;
+    m_msaaRenderBufferLegacy = nullptr;
+
+    delete m_msaaRenderTexture;
+    m_msaaRenderTexture = nullptr;
 
     delete m_msaaMultiViewRenderBuffer;
     m_msaaMultiViewRenderBuffer = nullptr;
@@ -301,16 +307,17 @@ QRhiTexture *QQuick3DSceneRenderer::renderToRhiTexture(QQuickWindow *qw)
 
         // Graphics pipeline objects depend on the MSAA sample count, so the
         // renderer needs to know the value.
-        rhiCtxD->setMainPassSampleCount(m_msaaRenderBuffer ? m_msaaRenderBuffer->sampleCount() :
-            (m_msaaMultiViewRenderBuffer ? m_msaaMultiViewRenderBuffer->sampleCount() : 1));
+        rhiCtxD->setMainPassSampleCount(m_msaaRenderBufferLegacy ? m_msaaRenderBufferLegacy->sampleCount() :
+            (m_msaaRenderTexture ? m_msaaRenderTexture->sampleCount() :
+                (m_msaaMultiViewRenderBuffer ? m_msaaMultiViewRenderBuffer->sampleCount() : 1)));
 
         // mainPassViewCount is left unchanged
 
         int ssaaAdjustedWidth = m_surfaceSize.width();
         int ssaaAdjustedHeight = m_surfaceSize.height();
         if (m_layer->antialiasingMode == QSSGRenderLayer::AAMode::SSAA) {
-            ssaaAdjustedWidth *= m_ssaaMultiplier;
-            ssaaAdjustedHeight *= m_ssaaMultiplier;
+            ssaaAdjustedWidth *= m_layer->ssaaMultiplier;
+            ssaaAdjustedHeight *= m_layer->ssaaMultiplier;
         }
 
         Q_TRACE(QSSG_prepareFrame_entry, ssaaAdjustedWidth, ssaaAdjustedHeight);
@@ -652,6 +659,37 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
     if (!m_layer)
         m_layer = new QSSGRenderLayer();
 
+    // Update the layer node properties
+    // Store the view count in the layer. If there are multiple, or nested views, sync is called multiple times and the view count
+    // can change (see: updateLayerNode()), so we need to store the value on the layer to make sure we don't end up with a mismatch
+    // between between the view count of the views rendering directly to the screen (XrView instance) and the view count of the offscreen
+    // rendered View3Ds.
+    // See also: preSynchronize(), queryMainRenderPassDescriptorAndCommandBuffer() and queryInlineRenderPassDescriptorAndCommandBuffer()
+    // (At this point the mainPassViewCount for this view should be set to the correct value)
+    m_layer->viewCount = rhiCtx->mainPassViewCount();
+    updateLayerNode(*m_layer, *view3D, resourceLoaders.values());
+
+    // Request extra frames for antialiasing (ProgressiveAA/TemporalAA)
+
+    m_requestedFramesCount = 0;
+    if (m_layer->isProgressiveAAEnabled()) {
+        // with progressive AA, we need a number of extra frames after the last dirty one
+        // if we always reset requestedFramesCount when dirty, we will get the extra frames eventually
+        // +1 since we need a normal frame to start with, and we're not copying that from the screen
+        m_requestedFramesCount = int(m_layer->antialiasingQuality) + 1;
+    } else if (m_layer->isTemporalAAEnabled()) {
+        // When temporalAA is on and antialiasing mode changes,
+        // layer needs to be re-rendered (at least) MAX_TEMPORAL_AA_LEVELS times
+        // to generate temporal antialiasing.
+        // Also, we need to do an extra render when animation stops
+        m_requestedFramesCount = (m_aaIsDirty || m_temporalIsDirty) ? QSSGLayerRenderData::MAX_TEMPORAL_AA_LEVELS : 1;
+    }
+
+    // Now that we have the effect list used for rendering, finalize the shader
+    // code based on the layer (scene.env.) settings.
+    for (QSSGRenderEffect *effectNode = m_layer->firstEffect; effectNode; effectNode = effectNode->m_nextEffect)
+        effectNode->finalizeShaders(*m_layer, m_sgContext.get());
+
     if (newRenderStats)
         m_renderStats->setRhiContext(rhiCtx, m_layer);
 
@@ -680,9 +718,6 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
         view3D->clearExtensionListDirty();
     }
 
-    // Update the layer node properties
-    updateLayerNode(view3D, resourceLoaders.values());
-
     bool postProcessingNeeded = m_layer->firstEffect;
     bool postProcessingWasActive = m_effectSystem;
     QSSGRenderTextureFormat::Format effectOutputFormatOverride = QSSGRenderTextureFormat::Unknown;
@@ -827,34 +862,9 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
         }
     }
 
-    const bool progressiveAA = m_layer->antialiasingMode == QSSGRenderLayer::AAMode::ProgressiveAA;
-    const bool multiSamplingAA = m_layer->antialiasingMode == QSSGRenderLayer::AAMode::MSAA;
-    const bool temporalAA = m_layer->temporalAAEnabled && !multiSamplingAA;
-    const bool superSamplingAA = m_layer->antialiasingMode == QSSGRenderLayer::AAMode::SSAA;
-    const bool timeBasedAA = progressiveAA || temporalAA;
-    m_postProcessingStack = m_layer->firstEffect || timeBasedAA  || superSamplingAA;
-    bool useFBO = view3D->renderMode() == QQuick3DViewport::RenderMode::Offscreen ||
-                                          ((view3D->renderMode() == QQuick3DViewport::RenderMode::Underlay || view3D->renderMode() == QQuick3DViewport::RenderMode::Overlay)
-                                           && m_postProcessingStack);
-
-    // Store the view count in the layer. If there's multiple, or nested views, sync is called multiple times and the view count
-    // can change, so we need to store the value on the lauer to make sure we don't end up with a mismatch between between
-    // the view count of the views rendering directly to the screen (XrView instance) and the view count of the offscreen rendered View3Ds.
-    // See also: preSynchronize(), queryMainRenderPassDescriptorAndCommandBuffer() and queryInlineRenderPassDescriptorAndCommandBuffer()
-    // (At this point the mainPassViewCount for this view should be set to the correct value)
-    m_layer->viewCount = rhiCtx->mainPassViewCount();
-
-    // NOTE: If we're rendering to an FBO, the view count is more then 1, and the View3D is not an XR view instance,
-    // we need to force the view count to 1 (The only time this should be the case is when embedding View3D(s)
-    // in XR with multiview enabled).
-    // Also, note that embedding View3D(s) in XR with multiview enabled only works if those View3D(s) are
-    // being rendered through a FBO.
-    if (useFBO && (m_layer->viewCount > 1) && !view3D->isXrViewInstance())
-        m_layer->viewCount = 1;
-
-    if (useFBO && rhiCtx->isValid()) {
+    if (m_useFBO && rhiCtx->isValid()) {
         QRhi *rhi = rhiCtx->rhi();
-        const QSize renderSize = superSamplingAA ? m_surfaceSize * m_ssaaMultiplier : m_surfaceSize;
+        const QSize renderSize = m_layer->isSsaaEnabled() ? m_surfaceSize * m_layer->ssaaMultiplier : m_surfaceSize;
 
         if (m_texture) {
             // the size changed, or the AA settings changed, or toggled between some effects - no effect
@@ -872,7 +882,7 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
                     // MSAA and texture for SSAA may need a different
                     // format now since m_texture's format could have
                     // changed between RBGA8 and RGBA16F (due to layerTextureFormat()).
-                    if (postProcessingStateDirty && (m_layer->antialiasingMode != QSSGRenderLayer::AAMode::NoAA || temporalAA)) {
+                    if (postProcessingStateDirty && (m_layer->antialiasingMode != QSSGRenderLayer::AAMode::NoAA || m_layer->isTemporalAAEnabled())) {
                         releaseAaDependentRhiResources();
                     } else {
                         if (m_ssaaTexture) {
@@ -887,9 +897,13 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
                             m_multiViewDepthStencilBuffer->setPixelSize(renderSize);
                             m_multiViewDepthStencilBuffer->create();
                         }
-                        if (m_msaaRenderBuffer) {
-                            m_msaaRenderBuffer->setPixelSize(renderSize);
-                            m_msaaRenderBuffer->create();
+                        if (m_msaaRenderBufferLegacy) {
+                            m_msaaRenderBufferLegacy->setPixelSize(renderSize);
+                            m_msaaRenderBufferLegacy->create();
+                        }
+                        if (m_msaaRenderTexture) {
+                            m_msaaRenderTexture->setPixelSize(renderSize);
+                            m_msaaRenderTexture->create();
                         }
                         if (m_msaaMultiViewRenderBuffer) {
                             m_msaaMultiViewRenderBuffer->setPixelSize(renderSize);
@@ -940,7 +954,7 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
             m_texture->create();
         }
 
-        if (!m_ssaaTexture && superSamplingAA) {
+        if (!m_ssaaTexture && m_layer->isSsaaEnabled()) {
             if (m_layer->viewCount >= 2)
                 m_ssaaTexture = rhi->newTextureArray(textureFormat, m_layer->viewCount, renderSize, 1, textureFlags);
             else
@@ -948,7 +962,7 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
             m_ssaaTexture->create();
         }
 
-        if (timeBasedAA && !m_temporalAATexture) {
+        if (m_timeBasedAA && !m_temporalAATexture) {
             m_temporalAATexture = rhi->newTexture(textureFormat, renderSize, 1, textureFlags);
             m_temporalAATexture->create();
             m_prevTempAATexture = rhi->newTexture(textureFormat, renderSize, 1, textureFlags);
@@ -990,7 +1004,8 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
 
         if (m_layer->viewCount >= 2) {
             if (!m_multiViewDepthStencilBuffer) {
-                m_multiViewDepthStencilBuffer = rhi->newTextureArray(QRhiTexture::D24S8, m_layer->viewCount, renderSize,
+                const auto format = rhi->isTextureFormatSupported(QRhiTexture::D24S8) ? QRhiTexture::D24S8 : QRhiTexture::D32FS8;
+                m_multiViewDepthStencilBuffer = rhi->newTextureArray(format, m_layer->viewCount, renderSize,
                                                                      m_samples, QRhiTexture::RenderTarget);
                 m_multiViewDepthStencilBuffer->create();
             }
@@ -1010,10 +1025,21 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
                     m_msaaMultiViewRenderBuffer->create();
                     att.setTexture(m_msaaMultiViewRenderBuffer);
                 } else {
-                    // pass in the texture's format (which may be a floating point one!) as the preferred format hint
-                    m_msaaRenderBuffer = rhi->newRenderBuffer(QRhiRenderBuffer::Color, renderSize, m_samples, {}, m_texture->format());
-                    m_msaaRenderBuffer->create();
-                    att.setRenderBuffer(m_msaaRenderBuffer);
+                    if (!rhi->isFeatureSupported(QRhi::MultisampleTexture)) {
+                        // pass in the texture's format (which may be a floating point one!) as the preferred format hint
+                        m_msaaRenderBufferLegacy = rhi->newRenderBuffer(QRhiRenderBuffer::Color, renderSize, m_samples, {}, m_texture->format());
+                        m_msaaRenderBufferLegacy->create();
+                        att.setRenderBuffer(m_msaaRenderBufferLegacy);
+                    } else {
+                        // The texture-backed MSAA path has the benefit of being able to use
+                        // GL_EXT_multisampled_render_to_texture on OpenGL ES when supported (Mali
+                        // and Qualcomm GPUs typically), potentially giving significant performance
+                        // gains. Whereas with 3D APIs other than OpenGL there is often no
+                        // difference between QRhiRenderBuffer and QRhiTexture anyway.
+                        m_msaaRenderTexture = rhi->newTexture(textureFormat, renderSize, m_samples, QRhiTexture::RenderTarget);
+                        m_msaaRenderTexture->create();
+                        att.setTexture(m_msaaRenderTexture);
+                    }
                 }
                 att.setResolveTexture(m_texture);
             } else {
@@ -1055,7 +1081,7 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
             m_effectSystem = nullptr;
         }
 
-        if (timeBasedAA && !m_temporalAARenderTarget) {
+        if (m_timeBasedAA && !m_temporalAARenderTarget) {
             m_temporalAARenderTarget = rhi->newTextureRenderTarget({ m_temporalAATexture });
             m_temporalAARenderTarget->setName(QByteArrayLiteral("Temporal AA texture"));
             m_temporalAARenderPassDescriptor = m_temporalAARenderTarget->newCompatibleRenderPassDescriptor();
@@ -1068,7 +1094,7 @@ void QQuick3DSceneRenderer::synchronize(QQuick3DViewport *view3D, const QSize &s
     }
 
     if (m_renderStats)
-        m_renderStats->endSync(dumpRenderTimes);
+        m_renderStats->endSync(dumpRenderTimes());
 
     Q_QUICK3D_PROFILE_END_WITH_ID(QQuick3DProfiler::Quick3DSynchronizeFrame, quint64(m_surfaceSize.width()) | quint64(m_surfaceSize.height()) << 32, profilingId);
 }
@@ -1167,36 +1193,55 @@ QQuick3DRenderStats *QQuick3DSceneRenderer::renderStats()
     return m_renderStats;
 }
 
-void QQuick3DRenderLayerHelpers::updateLayerNodeHelper(const QQuick3DViewport &view3D, QSSGRenderLayer &layerNode, bool &aaIsDirty, bool &temporalIsDirty, float &ssaaMultiplier)
+void QQuick3DRenderLayerHelpers::updateLayerNodeHelper(const QQuick3DViewport &view3D,
+                                                       const std::shared_ptr<QSSGRenderContextInterface>& rci,
+                                                       QSSGRenderLayer &layerNode,
+                                                       bool &aaIsDirty,
+                                                       bool &temporalIsDirty)
+{
+    QList<QSSGRenderGraphObject *> resourceLoaders; // empty list
+
+    QQuick3DSceneRenderer dummyRenderer(rci);
+
+    // Update the layer node properties
+    dummyRenderer.updateLayerNode(layerNode, view3D, resourceLoaders);
+
+    aaIsDirty = dummyRenderer.m_aaIsDirty;
+    temporalIsDirty = dummyRenderer.m_temporalIsDirty;
+}
+
+void QQuick3DSceneRenderer::updateLayerNode(QSSGRenderLayer &layerNode,
+                                            const QQuick3DViewport &view3D,
+                                            const QList<QSSGRenderGraphObject *> &resourceLoaders)
 {
     QQuick3DSceneEnvironment *environment = view3D.environment();
+    const auto &effects = environment->effectList();
 
     QSSGRenderLayer::AAMode aaMode = QSSGRenderLayer::AAMode(environment->antialiasingMode());
     if (aaMode != layerNode.antialiasingMode) {
         layerNode.antialiasingMode = aaMode;
         layerNode.progAAPassIndex = 0;
-        aaIsDirty = true;
+        m_aaIsDirty = true;
     }
     QSSGRenderLayer::AAQuality aaQuality = QSSGRenderLayer::AAQuality(environment->antialiasingQuality());
     if (aaQuality != layerNode.antialiasingQuality) {
         layerNode.antialiasingQuality = aaQuality;
-        ssaaMultiplier = (aaQuality == QSSGRenderLayer::AAQuality::Normal) ? 1.2f :
-                                                                               (aaQuality == QSSGRenderLayer::AAQuality::High) ? 1.5f :
-                                                                                                                                 2.0f;
-        layerNode.ssaaMultiplier = ssaaMultiplier;
-        aaIsDirty = true;
+        layerNode.ssaaMultiplier = QSSGRenderLayer::ssaaMultiplierForQuality(aaQuality);
+        m_aaIsDirty = true;
     }
 
-    bool temporalAAEnabled = environment->temporalAAEnabled();
-    if (temporalAAEnabled != layerNode.temporalAAEnabled) {
-        layerNode.temporalAAEnabled = environment->temporalAAEnabled();
-        temporalIsDirty = true;
+    // NOTE: Temporal AA is disabled when MSAA is enabled.
+    const bool temporalAARequested = environment->temporalAAEnabled();
+    const bool wasTaaEnabled = layerNode.isTemporalAAEnabled();
+    layerNode.temporalAAMode = temporalAARequested ? QSSGRenderLayer::TAAMode::On
+                                                   : QSSGRenderLayer::TAAMode::Off;
 
+    // If the state changed we need to reset the temporal AA pass index etc.
+    if (wasTaaEnabled != layerNode.isTemporalAAEnabled()) {
         layerNode.tempAAPassIndex = 0;
-        aaIsDirty = true;
+        m_aaIsDirty = true;
+        m_temporalIsDirty = true;
     }
-    layerNode.ssaaEnabled = environment->antialiasingMode()
-            == QQuick3DSceneEnvironment::QQuick3DEnvironmentAAModeValues::SSAA;
 
     layerNode.temporalAAStrength = environment->temporalAAStrength();
 
@@ -1234,13 +1279,7 @@ void QQuick3DRenderLayerHelpers::updateLayerNodeHelper(const QQuick3DViewport &v
     layerNode.lightProbeSettings.probeHorizon = qMin(environment->probeHorizon() - 1.0f, -0.001f);
     layerNode.setProbeOrientation(environment->probeOrientation());
 
-    layerNode.explicitCameras.clear();
-    if (!view3D.m_multiViewCameras.isEmpty()) {
-        for (QQuick3DCamera *camera : std::as_const(view3D.m_multiViewCameras))
-            layerNode.explicitCameras.append(static_cast<QSSGRenderCamera *>(QQuick3DObjectPrivate::get(camera)->spatialNode));
-    } else if (view3D.camera()) {
-        layerNode.explicitCameras.append(static_cast<QSSGRenderCamera *>(QQuick3DObjectPrivate::get(view3D.camera())->spatialNode));
-    }
+    QQuick3DViewport::updateCameraForLayer(view3D, layerNode);
 
     layerNode.layerFlags.setFlag(QSSGRenderLayer::LayerFlag::EnableDepthTest, environment->depthTestEnabled());
     layerNode.layerFlags.setFlag(QSSGRenderLayer::LayerFlag::EnableDepthPrePass, environment->depthPrePassEnabled());
@@ -1251,6 +1290,7 @@ void QQuick3DRenderLayerHelpers::updateLayerNodeHelper(const QQuick3DViewport &v
         layerNode.debugMode = QSSGRenderLayer::MaterialDebugMode(debugSettings->materialOverride());
         layerNode.wireframeMode = debugSettings->wireframeEnabled();
         layerNode.drawDirectionalLightShadowBoxes = debugSettings->drawDirectionalLightShadowBoxes();
+        layerNode.drawPointLightShadowBoxes = debugSettings->drawPointLightShadowBoxes();
         layerNode.drawShadowCastingBounds = debugSettings->drawShadowCastingBounds();
         layerNode.drawShadowReceivingBounds = debugSettings->drawShadowReceivingBounds();
         layerNode.drawCascades = debugSettings->drawCascades();
@@ -1293,57 +1333,55 @@ void QQuick3DRenderLayerHelpers::updateLayerNodeHelper(const QQuick3DViewport &v
     } else {
         layerNode.fog.enabled = false;
     }
-}
+    const auto method = static_cast<QSSGRenderLayer::OITMethod>(environment->oitMethod());
+    layerNode.oitMethodDirty = method != layerNode.oitMethod;
+    layerNode.oitMethod = method;
 
-void QQuick3DSceneRenderer::updateLayerNode(QQuick3DViewport *view3D, const QList<QSSGRenderGraphObject *> &resourceLoaders)
-{
-    QSSGRenderLayer *layerNode = m_layer;
-
-    bool temporalIsDirty = false;
-    QQuick3DRenderLayerHelpers::updateLayerNodeHelper(*view3D, *m_layer, m_aaIsDirty, temporalIsDirty, m_ssaaMultiplier);
-
-    int extraFramesToRender = 0;
-
-    if (layerNode->antialiasingMode == QSSGRenderLayer::AAMode::ProgressiveAA) {
-        // with progressive AA, we need a number of extra frames after the last dirty one
-        // if we always reset requestedFramesCount when dirty, we will get the extra frames eventually
-        // +1 since we need a normal frame to start with, and we're not copying that from the screen
-        extraFramesToRender = int(layerNode->antialiasingQuality) + 1;
-    } else if (layerNode->temporalAAEnabled) {
-        // When temporalAA is on and antialiasing mode changes,
-        // layer needs to be re-rendered (at least) MAX_TEMPORAL_AA_LEVELS times
-        // to generate temporal antialiasing.
-        // Also, we need to do an extra render when animation stops
-        extraFramesToRender = (m_aaIsDirty || temporalIsDirty) ? QSSGLayerRenderData::MAX_TEMPORAL_AA_LEVELS : 1;
-    }
-
-    requestedFramesCount = extraFramesToRender;
     // Effects need to be rendered in reverse order as described in the file.
-    layerNode->firstEffect = nullptr; // We reset the linked list
-    const auto &effects = view3D->environment()->effectList();
+    // NOTE: We only build up the list here, don't do anything that depends
+    //       on the collected layer state yet. See sync() for that.
+    layerNode.firstEffect = nullptr; // We reset the linked list
     auto rit = effects.crbegin();
     const auto rend = effects.crend();
     for (; rit != rend; ++rit) {
         QQuick3DObjectPrivate *p = QQuick3DObjectPrivate::get(*rit);
         QSSGRenderEffect *effectNode = static_cast<QSSGRenderEffect *>(p->spatialNode);
         if (effectNode) {
-            if (layerNode->hasEffect(effectNode)) {
+            if (layerNode.hasEffect(effectNode)) {
                 qWarning() << "Duplicate effect found, skipping!";
             } else {
                 effectNode->className = (*rit)->metaObject()->className(); //### persistent, but still icky to store a const char* returned from a function
-                layerNode->addEffect(*effectNode);
+                layerNode.addEffect(*effectNode);
             }
         }
     }
 
-    // Now that we have the effect list used for rendering, finalize the shader
-    // code based on the layer (scene.env.) settings.
-    for (QSSGRenderEffect *effectNode = layerNode->firstEffect; effectNode; effectNode = effectNode->m_nextEffect)
-        effectNode->finalizeShaders(*layerNode, m_sgContext.get());
+    const bool hasEffects = (layerNode.firstEffect != nullptr);
+
+    const auto renderMode = view3D.renderMode();
+
+    const bool progressiveAA = layerNode.isProgressiveAAEnabled();
+    const bool temporalAA = layerNode.isTemporalAAEnabled();
+    const bool superSamplingAA = layerNode.isSsaaEnabled();
+    m_timeBasedAA = progressiveAA || temporalAA;
+    m_postProcessingStack = hasEffects || m_timeBasedAA  || superSamplingAA;
+    m_useFBO = renderMode == QQuick3DViewport::RenderMode::Offscreen ||
+            ((renderMode == QQuick3DViewport::RenderMode::Underlay || renderMode == QQuick3DViewport::RenderMode::Overlay)
+             && m_postProcessingStack);
+
+    // Update the view count
+
+    // NOTE: If we're rendering to an FBO, the view count is more than 1, and the View3D is not an XR view instance,
+    // we need to force the view count to 1 (The only time this should be the case is when embedding View3D(s)
+    // in XR with multiview enabled).
+    // Also, note that embedding View3D(s) in XR with multiview enabled only works if those View3D(s) are
+    // being rendered through a FBO.
+    if (m_useFBO && (layerNode.viewCount > 1) && !view3D.isXrViewInstance())
+        layerNode.viewCount = 1;
 
     // ResourceLoaders
-    layerNode->resourceLoaders.clear();
-    layerNode->resourceLoaders = resourceLoaders;
+    layerNode.resourceLoaders.clear();
+    layerNode.resourceLoaders = resourceLoaders;
 }
 
 void QQuick3DSceneRenderer::removeNodeFromLayer(QSSGRenderNode *node)
@@ -1575,9 +1613,9 @@ void QQuick3DSGDirectRenderer::prepare()
                 queryMainRenderPassDescriptorAndCommandBuffer(m_window, m_renderer->m_sgContext->rhiContext().get());
                 const auto &quadRenderer = m_renderer->m_sgContext->renderer()->rhiQuadRenderer();
                 quadRenderer->prepareQuad(m_renderer->m_sgContext->rhiContext().get(), nullptr);
-                if (m_renderer->requestedFramesCount > 0) {
+                if (m_renderer->m_requestedFramesCount > 0) {
                     requestRender();
-                    m_renderer->requestedFramesCount--;
+                    m_renderer->m_requestedFramesCount--;
                 }
             }
         }
@@ -1671,7 +1709,7 @@ void QQuick3DSGDirectRenderer::render()
             m_renderer->endFrame();
 
             if (m_renderer->renderStats())
-                m_renderer->renderStats()->endRender(dumpRenderTimes);
+                m_renderer->renderStats()->endRender(dumpRenderTimes());
         }
     }
 }
