@@ -6,6 +6,7 @@
 
 #include <QtQuick3DRuntimeRender/private/qssgrenderdefaultmaterial_p.h>
 #include <QtQuick3DUtils/private/qssgutils_p.h>
+#include <QtQuick3D/private/qquick3dviewport_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -65,18 +66,15 @@ QT_BEGIN_NAMESPACE
 
     The default value is \c SpecularGlossyMaterial.FragmentLighting
 
-    When using \c SpecularGlossyMaterial.FragmentLighting, diffuse and specular lighting is
+    \value SpecularGlossyMaterial.FragmentLighting Diffuse and specular lighting is
     calculated for each rendered pixel. Certain effects (such as a Fresnel or normal map) require
     \c SpecularGlossyMaterial.FragmentLighting to work.
 
-    When using \c SpecularGlossyMaterial.NoLighting no lighting is calculated. This
+    \value SpecularGlossyMaterial.NoLighting No lighting is calculated. This
     mode is (predictably) very fast, and is quite effective when image maps are
     used that you do not need to be shaded by lighting. All other shading
     properties except albedo values, alpha values, and vertex colors will be
     ignored.
-
-    \value SpecularGlossyMaterial.NoLighting
-    \value SpecularGlossyMaterial.FragmentLighting
 */
 
 /*!
@@ -128,6 +126,29 @@ QT_BEGIN_NAMESPACE
     \sa albedo, alphaMode
 */
 
+/*!
+    \qmlproperty bool SpecularGlossyMaterial::albedoSingleChannelEnabled
+    \since 6.8
+
+    When this property is enabled, the material will use the single value of the albedoChannel from
+    the albedoMap as RGB value and use 1.0 as alpha value.
+    The default value is false.
+*/
+
+/*!
+    \qmlproperty enumeration SpecularGlossyMaterial::albedoChannel
+    \since 6.8
+
+    This property defines the texture channel used to read the albedo color value from albedoMap.
+    In order to use a single texture channel as color you have to enable the albedoSingleChannelEnabled
+    The default value is \c Material.R.
+
+    \value Material.R Read value from texture R channel.
+    \value Material.G Read value from texture G channel.
+    \value Material.B Read value from texture B channel.
+    \value Material.A Read value from texture A channel.
+*/
+
 /*!
     \qmlproperty color SpecularGlossyMaterial::specularColor
 
@@ -144,6 +165,29 @@ QT_BEGIN_NAMESPACE
     different parts of the material. Only the RGB channels are used.
 */
 
+/*!
+    \qmlproperty bool SpecularGlossyMaterial::specularSingleChannelEnabled
+    \since 6.8
+
+    When this property is enabled, the material will use the single value of the specularChannel from
+    the specularMap as RGB value.
+    The default value is false.
+*/
+
+/*!
+    \qmlproperty enumeration SpecularGlossyMaterial::specularChannel
+    \since 6.8
+
+    This property defines the texture channel used to read the specular color value from specularMap.
+    In order to use a single texture channel as color you have to enable the specularSingleChannelEnabled
+    The default value is \c Material.R.
+
+    \value Material.R Read value from texture R channel.
+    \value Material.G Read value from texture G channel.
+    \value Material.B Read value from texture B channel.
+    \value Material.A Read value from texture A channel.
+*/
+
 /*!
     \qmlproperty real SpecularGlossyMaterial::glossiness
 
@@ -173,6 +217,29 @@ QT_BEGIN_NAMESPACE
     \value Material.A Read value from texture A channel.
 */
 
+/*!
+    \qmlproperty bool SpecularGlossyMaterial::emissiveSingleChannelEnabled
+    \since 6.8
+
+    When this property is enabled, the material will use the single value of the emissiveChannel from
+    the emissiveMap as RGB value.
+    The default value is false.
+*/
+
+/*!
+    \qmlproperty enumeration SpecularGlossyMaterial::emissiveChannel
+    \since 6.8
+
+    This property defines the texture channel used to read the emissive color value from emissiveMap.
+    In order to use a single texture channel as color you have to enable the emissiveSingleChannelEnabled
+    The default value is \c Material.R.
+
+    \value Material.R Read value from texture R channel.
+    \value Material.G Read value from texture G channel.
+    \value Material.B Read value from texture B channel.
+    \value Material.A Read value from texture A channel.
+*/
+
 /*!
     \qmlproperty Texture SpecularGlossyMaterial::emissiveMap
 
@@ -206,6 +273,14 @@ QT_BEGIN_NAMESPACE
     different parts of the material.
 */
 
+/*!
+    \qmlproperty real SpecularGlossyMaterial::invertOpacityMapValue
+    \since 6.8
+
+    This property inverts the opacity value of the opacityMap.
+    The default value is \c false.
+*/
+
 /*!
     \qmlproperty enumeration SpecularGlossyMaterial::opacityChannel
 
@@ -423,7 +498,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::clearcoatAmount
+    \qmlproperty real SpecularGlossyMaterial::clearcoatAmount
 
     This property defines the intensity of the clearcoat layer.
 
@@ -453,7 +528,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::clearcoatRoughnessAmount
+    \qmlproperty real SpecularGlossyMaterial::clearcoatRoughnessAmount
 
     This property defines the roughness of the clearcoat layer.
     The default value is \c 0.0
@@ -491,7 +566,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::transmissionFactor
+    \qmlproperty real SpecularGlossyMaterial::transmissionFactor
 
     This property defines the percentage of light that is transmitted through
     the material's surface.
@@ -523,7 +598,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::thicknessFactor
+    \qmlproperty real SpecularGlossyMaterial::thicknessFactor
 
     This property defines the thickness of the volume beneath the surface.
     Unlike many other properties of SpecularGlossyMaterial, the value in defined
@@ -557,7 +632,7 @@ QT_BEGIN_NAMESPACE
 */
 
 /*!
-    \qmlproperty float SpecularGlossyMaterial::attenuationDistance
+    \qmlproperty real SpecularGlossyMaterial::attenuationDistance
 
     This property defines the Density of the medium given as the average
     distance that light travels in the medium before interacting with a
@@ -574,7 +649,170 @@ QT_BEGIN_NAMESPACE
 
 */
 
-inline static float ensureNormalized(float val) { return qBound(0.0f, val, 1.0f); }
+/*!
+    \qmlproperty real SpecularGlossyMaterial::fresnelScaleBiasEnabled
+
+     By Setting the value to true the material will take Fresnel Scale and Fresnel Bias into account.
+     The default value is \c false.
+*/
+
+/*!
+    \qmlproperty real SpecularGlossyMaterial::fresnelScale
+
+     This property scale head-on reflections (looking directly at the
+     surface) while maintaining reflections seen at grazing angles.
+     In order to affect changes to the material you have to enable fresnelScaleBiasEnabled.
+     The default value is \c 1.0.
+*/
+
+/*!
+    \qmlproperty real SpecularGlossyMaterial::fresnelBias
+
+     This property push forward head-on reflections (looking directly at the
+     surface) while maintaining reflections seen at grazing angles.
+     In order to affect changes to the material you have to enable fresnelScaleBiasEnabled.
+     The default value is \c 0.0.
+*/
+
+/*!
+    \qmlproperty real SpecularGlossyMaterial::fresnelPower
+
+     This property decreases head-on reflections (looking directly at the
+     surface) while maintaining reflections seen at grazing angles.
+     The default value is \c 5.0.
+*/
+
+/*!
+    \qmlproperty real SpecularGlossyMaterial::clearcoatFresnelScaleBiasEnabled
+
+    By Setting the value to true the material will take Clearcoat Fresnel Scale and Clearcoat Fresnel Bias into account.
+    The default value is \c false.
+*/
+
+/*!
+    \qmlproperty real SpecularGlossyMaterial::clearcoatFresnelScale
+
+    This property scale head-on reflections (looking directly at the
+    surface) while maintaining reflections seen at grazing angles.
+    In order to affect changes to the material you have to enable clearcoatFresnelScaleBiasEnabled.
+    The default value is \c 1.0.
+*/
+
+/*!
+    \qmlproperty real SpecularGlossyMaterial::clearcoatFresnelBias
+
+    This property push forward head-on reflections (looking directly at the
+    surface) while maintaining reflections seen at grazing angles.
+    In order to affect changes to the material you have to enable clearcoatFresnelScaleBiasEnabled.
+    The default value is \c 0.0.
+*/
+
+/*!
+    \qmlproperty real SpecularGlossyMaterial::clearcoatFresnelPower
+
+    This property decreases head-on reflections (looking directly at the
+    surface) while maintaining reflections seen at grazing angles.
+    The default value is \c 5.0.
+*/
+
+/*!
+    \qmlproperty bool SpecularGlossyMaterial::vertexColorsEnabled
+    \since 6.5
+
+    When this property is enabled, the material will use vertex colors from the
+    mesh. These will be multiplied by any other colors specified for the
+    material. The default value is true.
+*/
+
+/*!
+    \qmlproperty bool SpecularGlossyMaterial::vertexColorsMaskEnabled
+    \since 6.8
+
+    When this property is enabled, the material will use vertex colors from the
+    mesh as mask of various properties e.g Glossiness, OcclusionAmount, ... .
+    The default value is false.
+*/
+
+/*!
+    \qmlproperty enumeration SpecularGlossyMaterial::vertexColorRedMask
+    \since 6.8
+
+    This property defines the vertex color red channel used as the specifies mask.
+    The value is a bit-wise combination of flags.
+    The default value is \c SpecularGlossyMaterial.NoMask.
+
+    \value SpecularGlossyMaterial.NoMask.
+    \value SpecularGlossyMaterial.GlossinessMask.
+    \value SpecularGlossyMaterial.NormalStrengthMask.
+    \value SpecularGlossyMaterial.ClearcoatAmountMask.
+    \value SpecularGlossyMaterial.ClearcoatRoughnessAmountMask.
+    \value SpecularGlossyMaterial.ClearcoatNormalStrengthMask.
+    \value SpecularGlossyMaterial.HeightAmountMask.
+    \value SpecularGlossyMaterial.OcclusionAmountMask.
+    \value SpecularGlossyMaterial.ThicknessFactorMask.
+    \value SpecularGlossyMaterial.TransmissionFactorMask.
+
+*/
+
+/*!
+    \qmlproperty enumeration SpecularGlossyMaterial::vertexColorGreenMask
+    \since 6.8
+
+    This property defines the vertex color green channel used as the specifies mask.
+    The value is a bit-wise combination of flags.
+    The default value is \c SpecularGlossyMaterial.NoMask.
+
+    \value SpecularGlossyMaterial.NoMask.
+    \value SpecularGlossyMaterial.GlossinessMask.
+    \value SpecularGlossyMaterial.NormalStrengthMask.
+    \value SpecularGlossyMaterial.ClearcoatAmountMask.
+    \value SpecularGlossyMaterial.ClearcoatRoughnessAmountMask.
+    \value SpecularGlossyMaterial.ClearcoatNormalStrengthMask.
+    \value SpecularGlossyMaterial.HeightAmountMask.
+    \value SpecularGlossyMaterial.OcclusionAmountMask.
+    \value SpecularGlossyMaterial.ThicknessFactorMask.
+    \value SpecularGlossyMaterial.TransmissionFactorMask.
+*/
+
+/*!
+    \qmlproperty enumeration SpecularGlossyMaterial::vertexColorBlueMask
+    \since 6.8
+
+    This property defines the vertex color blue channel used as the specifies mask.
+    The value is a bit-wise combination of flags.
+    The default value is \c SpecularGlossyMaterial.NoMask.
+
+    \value SpecularGlossyMaterial.NoMask.
+    \value SpecularGlossyMaterial.GlossinessMask.
+    \value SpecularGlossyMaterial.NormalStrengthMask.
+    \value SpecularGlossyMaterial.ClearcoatAmountMask.
+    \value SpecularGlossyMaterial.ClearcoatRoughnessAmountMask.
+    \value SpecularGlossyMaterial.ClearcoatNormalStrengthMask.
+    \value SpecularGlossyMaterial.HeightAmountMask.
+    \value SpecularGlossyMaterial.OcclusionAmountMask.
+    \value SpecularGlossyMaterial.ThicknessFactorMask.
+    \value SpecularGlossyMaterial.TransmissionFactorMask.
+*/
+
+/*!
+    \qmlproperty enumeration SpecularGlossyMaterial::vertexColorAlphaMask
+    \since 6.8
+
+    This property defines the vertex color alpha channel used as the specifies mask.
+    The value is a bit-wise combination of flags.
+    The default value is \c SpecularGlossyMaterial.NoMask.
+
+    \value SpecularGlossyMaterial.NoMask.
+    \value SpecularGlossyMaterial.GlossinessMask.
+    \value SpecularGlossyMaterial.NormalStrengthMask.
+    \value SpecularGlossyMaterial.ClearcoatAmountMask.
+    \value SpecularGlossyMaterial.ClearcoatRoughnessAmountMask.
+    \value SpecularGlossyMaterial.ClearcoatNormalStrengthMask.
+    \value SpecularGlossyMaterial.HeightAmountMask.
+    \value SpecularGlossyMaterial.OcclusionAmountMask.
+    \value SpecularGlossyMaterial.ThicknessFactorMask.
+    \value SpecularGlossyMaterial.TransmissionFactorMask.
+*/
 
 QQuick3DSpecularGlossyMaterial::QQuick3DSpecularGlossyMaterial(QQuick3DObject *parent)
     : QQuick3DMaterial(*(new QQuick3DObjectPrivate(QQuick3DObjectPrivate::Type::SpecularGlossyMaterial)), parent)
@@ -604,6 +842,36 @@ QQuick3DTexture *QQuick3DSpecularGlossyMaterial::albedoMap() const
     return m_albedoMap;
 }
 
+bool QQuick3DSpecularGlossyMaterial::albedoSingleChannelEnabled() const
+{
+    return m_albedoSingleChannelEnabled;
+}
+
+QQuick3DMaterial::TextureChannelMapping QQuick3DSpecularGlossyMaterial::albedoChannel() const
+{
+    return m_albedoChannel;
+}
+
+bool QQuick3DSpecularGlossyMaterial::specularSingleChannelEnabled() const
+{
+    return m_specularSingleChannelEnabled;
+}
+
+QQuick3DMaterial::TextureChannelMapping QQuick3DSpecularGlossyMaterial::specularChannel() const
+{
+    return m_specularChannel;
+}
+
+bool QQuick3DSpecularGlossyMaterial::emissiveSingleChannelEnabled() const
+{
+    return m_emissiveSingleChannelEnabled;
+}
+
+QQuick3DMaterial::TextureChannelMapping QQuick3DSpecularGlossyMaterial::emissiveChannel() const
+{
+    return m_emissiveChannel;
+}
+
 QQuick3DTexture *QQuick3DSpecularGlossyMaterial::emissiveMap() const
 {
     return m_emissiveMap;
@@ -624,6 +892,11 @@ QQuick3DTexture *QQuick3DSpecularGlossyMaterial::glossinessMap() const
     return m_glossinessMap;
 }
 
+bool QQuick3DSpecularGlossyMaterial::invertOpacityMapValue() const
+{
+    return m_invertOpacityMapValue;
+}
+
 float QQuick3DSpecularGlossyMaterial::opacity() const
 {
     return m_opacity;
@@ -765,23 +1038,79 @@ void QQuick3DSpecularGlossyMaterial::setAlbedoMap(QQuick3DTexture *albedoMap)
     if (m_albedoMap == albedoMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(albedoMap, m_albedoMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("albedoMap"), m_connections, [this](QQuick3DObject *n) {
-        setAlbedoMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setAlbedoMap, albedoMap, m_albedoMap);
 
     m_albedoMap = albedoMap;
     emit albedoMapChanged();
     markDirty(AlbedoDirty);
 }
 
+void QQuick3DSpecularGlossyMaterial::setAlbedoSingleChannelEnabled(bool albedoSingleChannelEnabled)
+{
+    if (m_albedoSingleChannelEnabled == albedoSingleChannelEnabled)
+        return;
+
+    m_albedoSingleChannelEnabled = albedoSingleChannelEnabled;
+    emit albedoSingleChannelEnabledChanged();
+    markDirty(AlbedoDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setAlbedoChannel(TextureChannelMapping channel)
+{
+    if (m_albedoChannel == channel)
+        return;
+
+    m_albedoChannel = channel;
+    emit albedoChannelChanged();
+    markDirty(AlbedoDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setSpecularSingleChannelEnabled(bool specularSingleChannelEnabled)
+{
+    if (m_specularSingleChannelEnabled == specularSingleChannelEnabled)
+        return;
+
+    m_specularSingleChannelEnabled = specularSingleChannelEnabled;
+    emit specularSingleChannelEnabledChanged();
+    markDirty(SpecularDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setSpecularChannel(TextureChannelMapping channel)
+{
+    if (m_specularChannel == channel)
+        return;
+
+    m_specularChannel = channel;
+    emit specularChannelChanged();
+    markDirty(SpecularDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setEmissiveSingleChannelEnabled(bool emissiveSingleChannelEnabled)
+{
+    if (m_emissiveSingleChannelEnabled == emissiveSingleChannelEnabled)
+        return;
+
+    m_emissiveSingleChannelEnabled = emissiveSingleChannelEnabled;
+    emit emissiveSingleChannelEnabledChanged();
+    markDirty(EmissiveDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setEmissiveChannel(TextureChannelMapping channel)
+{
+    if (m_emissiveChannel == channel)
+        return;
+
+    m_emissiveChannel = channel;
+    emit emissiveChannelChanged();
+    markDirty(EmissiveDirty);
+}
+
 void QQuick3DSpecularGlossyMaterial::setEmissiveMap(QQuick3DTexture *emissiveMap)
 {
     if (m_emissiveMap == emissiveMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(emissiveMap, m_emissiveMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("emissiveMap"), m_connections, [this](QQuick3DObject *n) {
-        setEmissiveMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setEmissiveMap, emissiveMap, m_emissiveMap);
 
     m_emissiveMap = emissiveMap;
     emit emissiveMapChanged();
@@ -814,15 +1143,23 @@ void QQuick3DSpecularGlossyMaterial::setGlossinessMap(QQuick3DTexture *glossines
     if (m_glossinessMap == glossinessMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(glossinessMap, m_glossinessMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("glossinessMap"), m_connections, [this](QQuick3DObject *n) {
-        setGlossinessMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setGlossinessMap, glossinessMap, m_glossinessMap);
 
     m_glossinessMap = glossinessMap;
     emit glossinessMapChanged();
     markDirty(GlossyDirty);
 }
 
+void QQuick3DSpecularGlossyMaterial::setInvertOpacityMapValue(bool invertOpacityMapValue)
+{
+    if (invertOpacityMapValue == m_invertOpacityMapValue)
+        return;
+
+    m_invertOpacityMapValue = invertOpacityMapValue;
+    emit invertOpacityMapValueChanged();
+    markDirty(OpacityDirty);
+}
+
 void QQuick3DSpecularGlossyMaterial::setOpacity(float opacity)
 {
     opacity = ensureNormalized(opacity);
@@ -839,9 +1176,7 @@ void QQuick3DSpecularGlossyMaterial::setOpacityMap(QQuick3DTexture *opacityMap)
     if (m_opacityMap == opacityMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(opacityMap, m_opacityMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("opacityMap"), m_connections, [this](QQuick3DObject *n) {
-        setOpacityMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setOpacityMap, opacityMap, m_opacityMap);
 
     m_opacityMap = opacityMap;
     emit opacityMapChanged();
@@ -853,9 +1188,7 @@ void QQuick3DSpecularGlossyMaterial::setNormalMap(QQuick3DTexture *normalMap)
     if (m_normalMap == normalMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(normalMap, m_normalMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("normalMap"), m_connections, [this](QQuick3DObject *n) {
-        setNormalMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setNormalMap, normalMap, m_normalMap);
 
     m_normalMap = normalMap;
     emit normalMapChanged();
@@ -877,9 +1210,7 @@ void QQuick3DSpecularGlossyMaterial::setSpecularMap(QQuick3DTexture *specularMap
     if (m_specularMap == specularMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(specularMap, m_specularMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("specularMap"), m_connections, [this](QQuick3DObject *n) {
-        setSpecularMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setSpecularMap, specularMap, m_specularMap);
 
     m_specularMap = specularMap;
     emit specularMapChanged();
@@ -902,9 +1233,7 @@ void QQuick3DSpecularGlossyMaterial::setOcclusionMap(QQuick3DTexture *occlusionM
     if (m_occlusionMap == occlusionMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(occlusionMap, m_occlusionMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("occlusionMap"), m_connections, [this](QQuick3DObject *n) {
-        setOcclusionMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setOcclusionMap, occlusionMap, m_occlusionMap);
 
     m_occlusionMap = occlusionMap;
     emit occlusionMapChanged();
@@ -994,9 +1323,7 @@ void QQuick3DSpecularGlossyMaterial::setHeightMap(QQuick3DTexture *heightMap)
     if (m_heightMap == heightMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(heightMap, m_heightMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("heightMap"), m_connections, [this](QQuick3DObject *n) {
-        setHeightMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setHeightMap, heightMap, m_heightMap);
 
     m_heightMap = heightMap;
     emit heightMapChanged();
@@ -1073,7 +1400,9 @@ QSSGRenderGraphObject *QQuick3DSpecularGlossyMaterial::updateSpatialNode(QSSGRen
         else
             material->colorMap = m_albedoMap->getRenderImage();
 
-        material->color = color::sRGBToLinear(m_albedo);
+        material->color = QSSGUtils::color::sRGBToLinear(m_albedo);
+        material->baseColorSingleChannelEnabled = m_albedoSingleChannelEnabled;
+        material->baseColorChannel = channelMapping(m_albedoChannel);
     }
 
     if (m_dirtyAttributes & EmissiveDirty) {
@@ -1083,11 +1412,10 @@ QSSGRenderGraphObject *QQuick3DSpecularGlossyMaterial::updateSpatialNode(QSSGRen
             material->emissiveMap = m_emissiveMap->getRenderImage();
 
         material->emissiveColor = m_emissiveFactor;
+        material->emissiveSingleChannelEnabled = m_emissiveSingleChannelEnabled;
+        material->emissiveChannel = channelMapping(m_emissiveChannel);
     }
 
-    material->fresnelPower = 5.0f;
-    material->vertexColorsEnabled = false;
-
     if (m_dirtyAttributes & GlossyDirty) {
         if (!m_glossinessMap)
             material->roughnessMap = nullptr;
@@ -1104,7 +1432,13 @@ QSSGRenderGraphObject *QQuick3DSpecularGlossyMaterial::updateSpatialNode(QSSGRen
         else
             material->specularMap = m_specularMap->getRenderImage();
 
-        material->specularTint = color::sRGBToLinear(m_specular).toVector3D();
+        material->specularTint = QSSGUtils::color::sRGBToLinear(m_specular).toVector3D();
+        material->fresnelScaleBiasEnabled = m_fresnelScaleBiasEnabled;
+        material->fresnelScale = m_fresnelScale;
+        material->fresnelBias = m_fresnelBias;
+        material->fresnelPower = m_fresnelPower;
+        material->specularAmountSingleChannelEnabled = m_specularSingleChannelEnabled;
+        material->specularAmountChannel = channelMapping(m_specularChannel);
     }
 
     if (m_dirtyAttributes & OpacityDirty) {
@@ -1113,6 +1447,7 @@ QSSGRenderGraphObject *QQuick3DSpecularGlossyMaterial::updateSpatialNode(QSSGRen
         else
             material->opacityMap = m_opacityMap->getRenderImage();
 
+        material->invertOpacityMapValue = m_invertOpacityMapValue;
         material->opacity = m_opacity;
         material->opacityChannel = channelMapping(m_opacityChannel);
     }
@@ -1174,6 +1509,11 @@ QSSGRenderGraphObject *QQuick3DSpecularGlossyMaterial::updateSpatialNode(QSSGRen
             material->clearcoatNormalMap = nullptr;
         else
             material->clearcoatNormalMap = m_clearcoatNormalMap->getRenderImage();
+        material->clearcoatNormalStrength = m_clearcoatNormalStrength;
+        material->clearcoatFresnelScaleBiasEnabled = m_clearcoatFresnelScaleBiasEnabled;
+        material->clearcoatFresnelScale = m_clearcoatFresnelScale;
+        material->clearcoatFresnelBias = m_clearcoatFresnelBias;
+        material->clearcoatFresnelPower = m_clearcoatFresnelPower;
     }
 
     if (m_dirtyAttributes & TransmissionDirty) {
@@ -1194,7 +1534,16 @@ QSSGRenderGraphObject *QQuick3DSpecularGlossyMaterial::updateSpatialNode(QSSGRen
         material->thicknessChannel = channelMapping(m_thicknessChannel);
 
         material->attenuationDistance = m_attenuationDistance;
-        material->attenuationColor = color::sRGBToLinear(m_attenuationColor).toVector3D();
+        material->attenuationColor = QSSGUtils::color::sRGBToLinear(m_attenuationColor).toVector3D();
+    }
+
+    if (m_dirtyAttributes & VertexColorsDirty) {
+        material->vertexColorsEnabled = m_vertexColorsEnabled;
+        material->vertexColorsMaskEnabled = m_vertexColorsMaskEnabled;
+        material->vertexColorRedMask = QSSGRenderDefaultMaterial::VertexColorMaskFlags::fromInt(m_vertexColorRedMask);
+        material->vertexColorGreenMask = QSSGRenderDefaultMaterial::VertexColorMaskFlags::fromInt(m_vertexColorGreenMask);
+        material->vertexColorBlueMask = QSSGRenderDefaultMaterial::VertexColorMaskFlags::fromInt(m_vertexColorBlueMask);
+        material->vertexColorAlphaMask = QSSGRenderDefaultMaterial::VertexColorMaskFlags::fromInt(m_vertexColorAlphaMask);
     }
 
     m_dirtyAttributes = 0;
@@ -1274,9 +1623,7 @@ void QQuick3DSpecularGlossyMaterial::setClearcoatMap(QQuick3DTexture *newClearco
     if (m_clearcoatMap == newClearcoatMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(newClearcoatMap, m_clearcoatMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("clearcoatMap"), m_connections, [this](QQuick3DObject *n) {
-        setClearcoatMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setClearcoatMap, newClearcoatMap, m_clearcoatMap);
 
     m_clearcoatMap = newClearcoatMap;
     emit clearcoatMapChanged();
@@ -1335,9 +1682,7 @@ void QQuick3DSpecularGlossyMaterial::setClearcoatRoughnessMap(QQuick3DTexture *n
     if (m_clearcoatRoughnessMap == newClearcoatRoughnessMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(newClearcoatRoughnessMap, m_clearcoatRoughnessMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("clearcoatRoughnessMap"), m_connections, [this](QQuick3DObject *n) {
-        setClearcoatRoughnessMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setClearcoatRoughnessMap, newClearcoatRoughnessMap, m_clearcoatRoughnessMap);
 
     m_clearcoatRoughnessMap = newClearcoatRoughnessMap;
     emit clearcoatRoughnessMapChanged();
@@ -1354,15 +1699,28 @@ void QQuick3DSpecularGlossyMaterial::setClearcoatNormalMap(QQuick3DTexture *newC
     if (m_clearcoatNormalMap == newClearcoatNormalMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(newClearcoatNormalMap, m_clearcoatNormalMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("clearcoatNormalMap"), m_connections, [this](QQuick3DObject *n) {
-        setClearcoatNormalMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setClearcoatNormalMap, newClearcoatNormalMap, m_clearcoatNormalMap);
 
     m_clearcoatNormalMap = newClearcoatNormalMap;
     emit clearcoatNormalMapChanged();
     markDirty(ClearcoatDirty);
 }
 
+float QQuick3DSpecularGlossyMaterial::clearcoatNormalStrength() const
+{
+    return m_clearcoatNormalStrength;
+}
+
+
+void QQuick3DSpecularGlossyMaterial::setClearcoatNormalStrength(float newClearcoatNormalStrength)
+{
+    if (qFuzzyCompare(m_clearcoatNormalStrength, newClearcoatNormalStrength))
+        return;
+    m_clearcoatNormalStrength = newClearcoatNormalStrength;
+    emit clearcoatNormalStrengthChanged();
+    markDirty(ClearcoatDirty);
+}
+
 float QQuick3DSpecularGlossyMaterial::transmissionFactor() const
 {
     return m_transmissionFactor;
@@ -1387,9 +1745,7 @@ void QQuick3DSpecularGlossyMaterial::setTransmissionMap(QQuick3DTexture *newTran
     if (m_transmissionMap == newTransmissionMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(newTransmissionMap, m_transmissionMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("transmissionMap"), m_connections, [this](QQuick3DObject *n) {
-        setTransmissionMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setTransmissionMap, newTransmissionMap, m_transmissionMap);
 
     m_transmissionMap = newTransmissionMap;
     emit transmissionMapChanged();
@@ -1434,9 +1790,7 @@ void QQuick3DSpecularGlossyMaterial::setThicknessMap(QQuick3DTexture *newThickne
     if (m_thicknessMap == newThicknessMap)
         return;
 
-    QQuick3DObjectPrivate::updatePropertyListener(newThicknessMap, m_thicknessMap, QQuick3DObjectPrivate::get(this)->sceneManager, QByteArrayLiteral("thicknessMap"), m_connections, [this](QQuick3DObject *n) {
-        setThicknessMap(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DSpecularGlossyMaterial::setThicknessMap, newThicknessMap, m_thicknessMap);
 
     m_thicknessMap = newThicknessMap;
     emit thicknessMapChanged();
@@ -1476,6 +1830,51 @@ QColor QQuick3DSpecularGlossyMaterial::attenuationColor() const
     return m_attenuationColor;
 }
 
+bool QQuick3DSpecularGlossyMaterial::fresnelScaleBiasEnabled() const
+{
+    return m_fresnelScaleBiasEnabled;
+}
+
+float QQuick3DSpecularGlossyMaterial::fresnelScale() const
+{
+    return m_fresnelScale;
+}
+
+float QQuick3DSpecularGlossyMaterial::fresnelBias() const
+{
+    return m_fresnelBias;
+}
+
+float QQuick3DSpecularGlossyMaterial::fresnelPower() const
+{
+    return m_fresnelPower;
+}
+
+bool QQuick3DSpecularGlossyMaterial::clearcoatFresnelScaleBiasEnabled() const
+{
+    return m_clearcoatFresnelScaleBiasEnabled;
+}
+
+float QQuick3DSpecularGlossyMaterial::clearcoatFresnelScale() const
+{
+    return m_clearcoatFresnelScale;
+}
+
+float QQuick3DSpecularGlossyMaterial::clearcoatFresnelBias() const
+{
+    return m_clearcoatFresnelBias;
+}
+
+float QQuick3DSpecularGlossyMaterial::clearcoatFresnelPower() const
+{
+    return m_clearcoatFresnelPower;
+}
+
+bool QQuick3DSpecularGlossyMaterial::vertexColorsEnabled() const
+{
+    return m_vertexColorsEnabled;
+}
+
 void QQuick3DSpecularGlossyMaterial::setAttenuationColor(const QColor &newAttenuationColor)
 {
     if (m_attenuationColor == newAttenuationColor)
@@ -1485,4 +1884,164 @@ void QQuick3DSpecularGlossyMaterial::setAttenuationColor(const QColor &newAttenu
     markDirty(VolumeDirty);
 }
 
+void QQuick3DSpecularGlossyMaterial::setFresnelScaleBiasEnabled(bool fresnelScaleBiasEnabled)
+{
+    if (m_fresnelScaleBiasEnabled == fresnelScaleBiasEnabled)
+        return;
+
+    m_fresnelScaleBiasEnabled = fresnelScaleBiasEnabled;
+    emit fresnelScaleBiasEnabledChanged(m_fresnelScaleBiasEnabled);
+    markDirty(SpecularDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setFresnelScale(float fresnelScale)
+{
+    if (qFuzzyCompare(m_fresnelScale, fresnelScale))
+        return;
+
+    m_fresnelScale = fresnelScale;
+    emit fresnelScaleChanged(m_fresnelScale);
+    markDirty(SpecularDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setFresnelBias(float fresnelBias)
+{
+    if (qFuzzyCompare(m_fresnelBias, fresnelBias))
+        return;
+
+    m_fresnelBias = fresnelBias;
+    emit fresnelBiasChanged(m_fresnelBias);
+    markDirty(SpecularDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setFresnelPower(float fresnelPower)
+{
+    if (qFuzzyCompare(m_fresnelPower, fresnelPower))
+        return;
+
+    m_fresnelPower = fresnelPower;
+    emit fresnelPowerChanged(m_fresnelPower);
+    markDirty(SpecularDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setClearcoatFresnelScaleBiasEnabled(bool clearcoatFresnelScaleBiasEnabled)
+{
+    if (m_clearcoatFresnelScaleBiasEnabled == clearcoatFresnelScaleBiasEnabled)
+        return;
+
+    m_clearcoatFresnelScaleBiasEnabled = clearcoatFresnelScaleBiasEnabled;
+    emit clearcoatFresnelScaleBiasEnabledChanged(m_clearcoatFresnelScaleBiasEnabled);
+    markDirty(ClearcoatDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setClearcoatFresnelScale(float clearcoatFresnelScale)
+{
+    if (qFuzzyCompare(m_clearcoatFresnelScale, clearcoatFresnelScale))
+        return;
+
+    m_clearcoatFresnelScale = clearcoatFresnelScale;
+    emit clearcoatFresnelScaleChanged(m_clearcoatFresnelScale);
+    markDirty(ClearcoatDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setClearcoatFresnelBias(float clearcoatFresnelBias)
+{
+    if (qFuzzyCompare(m_clearcoatFresnelBias, clearcoatFresnelBias))
+        return;
+
+    m_clearcoatFresnelBias = clearcoatFresnelBias;
+    emit clearcoatFresnelBiasChanged(m_clearcoatFresnelBias);
+    markDirty(ClearcoatDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setClearcoatFresnelPower(float clearcoatFresnelPower)
+{
+    if (qFuzzyCompare(m_clearcoatFresnelPower, clearcoatFresnelPower))
+        return;
+
+    m_clearcoatFresnelPower = clearcoatFresnelPower;
+    emit clearcoatFresnelPowerChanged(m_clearcoatFresnelPower);
+    markDirty(ClearcoatDirty);
+}
+
+void QQuick3DSpecularGlossyMaterial::setVertexColorsEnabled(bool vertexColors)
+{
+    if (m_vertexColorsEnabled == vertexColors)
+        return;
+
+    m_vertexColorsEnabled = vertexColors;
+    emit vertexColorsEnabledChanged(m_vertexColorsEnabled);
+    markDirty(VertexColorsDirty);
+}
+
+bool QQuick3DSpecularGlossyMaterial::vertexColorsMaskEnabled() const
+{
+    return m_vertexColorsMaskEnabled;
+}
+
+void QQuick3DSpecularGlossyMaterial::setVertexColorsMaskEnabled(bool vertexColorsMaskEnabled)
+{
+    if (m_vertexColorsMaskEnabled == vertexColorsMaskEnabled)
+        return;
+    m_vertexColorsMaskEnabled = vertexColorsMaskEnabled;
+    emit vertexColorsMaskEnabledChanged();
+    markDirty(VertexColorsDirty);
+}
+
+QQuick3DSpecularGlossyMaterial::VertexColorMaskFlags QQuick3DSpecularGlossyMaterial::vertexColorRedMask() const
+{
+    return m_vertexColorRedMask;
+}
+
+void QQuick3DSpecularGlossyMaterial::setVertexColorRedMask(QQuick3DSpecularGlossyMaterial::VertexColorMaskFlags vertexColorRedMask)
+{
+    if (m_vertexColorRedMask == vertexColorRedMask)
+        return;
+    m_vertexColorRedMask = vertexColorRedMask;
+    emit vertexColorRedMaskChanged();
+    markDirty(VertexColorsDirty);
+}
+
+QQuick3DSpecularGlossyMaterial::VertexColorMaskFlags QQuick3DSpecularGlossyMaterial::vertexColorGreenMask() const
+{
+    return m_vertexColorGreenMask;
+}
+
+void QQuick3DSpecularGlossyMaterial::setVertexColorGreenMask(QQuick3DSpecularGlossyMaterial::VertexColorMaskFlags vertexColorGreenMask)
+{
+    if (m_vertexColorGreenMask == vertexColorGreenMask)
+        return;
+    m_vertexColorGreenMask = vertexColorGreenMask;
+    emit vertexColorGreenMaskChanged();
+    markDirty(VertexColorsDirty);
+}
+
+QQuick3DSpecularGlossyMaterial::VertexColorMaskFlags QQuick3DSpecularGlossyMaterial::vertexColorBlueMask() const
+{
+    return m_vertexColorBlueMask;
+}
+
+void QQuick3DSpecularGlossyMaterial::setVertexColorBlueMask(QQuick3DSpecularGlossyMaterial::VertexColorMaskFlags vertexColorBlueMask)
+{
+    if (m_vertexColorBlueMask == vertexColorBlueMask)
+        return;
+    m_vertexColorBlueMask = vertexColorBlueMask;
+    emit vertexColorBlueMaskChanged();
+    markDirty(VertexColorsDirty);
+}
+
+QQuick3DSpecularGlossyMaterial::VertexColorMaskFlags QQuick3DSpecularGlossyMaterial::vertexColorAlphaMask() const
+{
+    return m_vertexColorAlphaMask;
+}
+
+void QQuick3DSpecularGlossyMaterial::setVertexColorAlphaMask(QQuick3DSpecularGlossyMaterial::VertexColorMaskFlags vertexColorAlphaMask)
+{
+    if (m_vertexColorAlphaMask == vertexColorAlphaMask)
+        return;
+    m_vertexColorAlphaMask = vertexColorAlphaMask;
+    emit vertexColorAlphaMaskChanged();
+    markDirty(VertexColorsDirty);
+}
+
 QT_END_NAMESPACE
