@@ -39,9 +39,8 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_LOGGING_CATEGORY(lcEv, "qt.quick3d.event")
-Q_LOGGING_CATEGORY(lcPick, "qt.quick3d.pick")
-Q_LOGGING_CATEGORY(lcHover, "qt.quick3d.hover")
+Q_STATIC_LOGGING_CATEGORY(lcEv, "qt.quick3d.event")
+Q_STATIC_LOGGING_CATEGORY(lcPick, "qt.quick3d.pick")
 
 static bool isforceInputHandlingSet()
 {
@@ -1874,7 +1873,7 @@ bool QQuick3DViewport::singlePointPick(QSinglePointEvent *event, const QVector3D
     for (const auto &pickResult : pickResults) {
         auto [item, position] = getItemAndPosition(pickResult);
         if (!item)
-            continue;
+            break;
         if (item == m_prevMouseItem && (position - m_prevMousePos).manhattanLength() < jitterLimit && !event->button()) {
             withinJitterLimit = true;
             break;
