@@ -24,13 +24,13 @@ QT_BEGIN_NAMESPACE
 QQuick3DParticleSpriteParticle::QQuick3DParticleSpriteParticle(QQuick3DNode *parent)
     : QQuick3DParticle(parent)
 {
-    m_connections.insert("maxAmount", QObject::connect(this, &QQuick3DParticle::maxAmountChanged, [this]() {
+    m_connections.insert("maxAmount", QObject::connect(this, &QQuick3DParticle::maxAmountChanged, this, [this]() {
         handleMaxAmountChanged(m_maxAmount);
     }));
-    m_connections.insert("system", QObject::connect(this, &QQuick3DParticle::systemChanged, [this]() {
+    m_connections.insert("system", QObject::connect(this, &QQuick3DParticle::systemChanged, this, [this]() {
         handleSystemChanged(system());
     }));
-    m_connections.insert("sortMode", QObject::connect(this, &QQuick3DParticle::sortModeChanged, [this]() {
+    m_connections.insert("sortMode", QObject::connect(this, &QQuick3DParticle::sortModeChanged, this, [this]() {
         markNodesDirty();
     }));
 }
@@ -185,7 +185,7 @@ QQmlListProperty<QQuick3DAbstractLight> QQuick3DParticleSpriteParticle::lights()
 }
 
 /*!
-    \qmlproperty float SpriteParticle3D::offsetX
+    \qmlproperty real SpriteParticle3D::offsetX
     \since 6.3
 
     This property defines the particles offset in the X axis
@@ -196,7 +196,7 @@ float QQuick3DParticleSpriteParticle::offsetX() const
 }
 
 /*!
-    \qmlproperty float SpriteParticle3D::offsetY
+    \qmlproperty real SpriteParticle3D::offsetY
     \since 6.3
 
     This property defines the particles offset in the Y axis
@@ -232,12 +232,7 @@ void QQuick3DParticleSpriteParticle::setSprite(QQuick3DTexture *sprite)
     if (m_sprite == sprite)
         return;
 
-    auto sceneManager = QQuick3DObjectPrivate::get(this)->sceneManager;
-    QQuick3DObjectPrivate::updatePropertyListener(sprite, m_sprite, sceneManager,
-                                                  QByteArrayLiteral("sprite"), m_connections,
-                                                  [this](QQuick3DObject *n) {
-        setSprite(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DParticleSpriteParticle::setSprite, sprite, m_sprite);
 
     m_sprite = sprite;
     markNodesDirty();
@@ -278,12 +273,7 @@ void QQuick3DParticleSpriteParticle::setColorTable(QQuick3DTexture *colorTable)
     if (m_colorTable == colorTable)
         return;
 
-    auto sceneManager = QQuick3DObjectPrivate::get(this)->sceneManager;
-    QQuick3DObjectPrivate::updatePropertyListener(colorTable, m_colorTable, sceneManager,
-                                                  QByteArrayLiteral("colorTable"), m_connections,
-                                                  [this](QQuick3DObject *n) {
-        setColorTable(qobject_cast<QQuick3DTexture *>(n));
-    });
+    QQuick3DObjectPrivate::attachWatcher(this, &QQuick3DParticleSpriteParticle::setColorTable, colorTable, m_colorTable);
 
     m_colorTable = colorTable;
     updateFeatureLevel();
@@ -333,10 +323,9 @@ static QSSGRenderParticles::BlendMode mapBlendMode(QQuick3DParticleSpriteParticl
         return QSSGRenderParticles::BlendMode::Screen;
     case QQuick3DParticleSpriteParticle::Multiply:
         return QSSGRenderParticles::BlendMode::Multiply;
-    default:
-        Q_ASSERT(false);
-        return QSSGRenderParticles::BlendMode::SourceOver;
     }
+
+    Q_UNREACHABLE_RETURN(QSSGRenderParticles::BlendMode::SourceOver);
 }
 
 QSSGRenderParticles::FeatureLevel QQuick3DParticleSpriteParticle::mapFeatureLevel(QQuick3DParticleSpriteParticle::FeatureLevel level)
@@ -354,10 +343,9 @@ QSSGRenderParticles::FeatureLevel QQuick3DParticleSpriteParticle::mapFeatureLeve
         return QSSGRenderParticles::FeatureLevel::MappedVLight;
     case QQuick3DParticleSpriteParticle::AnimatedVLight:
         return QSSGRenderParticles::FeatureLevel::AnimatedVLight;
-    default:
-        Q_ASSERT(false);
-        return QSSGRenderParticles::FeatureLevel::Simple;
     }
+
+    Q_UNREACHABLE_RETURN(QSSGRenderParticles::FeatureLevel::Simple);
 }
 
 QSSGRenderGraphObject *QQuick3DParticleSpriteParticle::ParticleUpdateNode::updateSpatialNode(QSSGRenderGraphObject *node)
@@ -365,7 +353,7 @@ QSSGRenderGraphObject *QQuick3DParticleSpriteParticle::ParticleUpdateNode::updat
     if (m_particle) {
         node = m_particle->updateParticleNode(this, node);
         QQuick3DNode::updateSpatialNode(node);
-
+        Q_QUICK3D_PROFILE_ASSIGN_ID_SG(m_particle, node);
         auto particles = static_cast<QSSGRenderParticles *>(node);
 
         if (m_particle->m_featureLevel == QQuick3DParticleSpriteParticle::Animated || m_particle->m_featureLevel == QQuick3DParticleSpriteParticle::AnimatedVLight)
@@ -446,7 +434,7 @@ QSSGRenderGraphObject *QQuick3DParticleSpriteParticle::updateParticleNode(const
 
     particles->m_blendMode = mapBlendMode(m_blendMode);
     particles->m_billboard = m_billboard;
-    particles->m_depthBias = perEmitter.emitter->depthBias();
+    particles->m_depthBiasSq = QSSGRenderNode::signedSquared(perEmitter.emitter->depthBias());
     particles->m_featureLevel = mapFeatureLevel(m_featureLevel);
     particles->m_depthSorting = sortMode() == QQuick3DParticle::SortDistance;
     particles->m_castsReflections = m_castsReflections;
@@ -778,7 +766,7 @@ void QQuick3DParticleSpriteParticle::qmlClearLights(QQmlListProperty<QQuick3DAbs
     for (const auto &light : std::as_const(self->m_lights)) {
         if (light->parentItem() == nullptr)
             QQuick3DObjectPrivate::get(light)->derefSceneManager();
-        light->disconnect(self, SLOT(onLightDestroyed(QObject*)));
+        disconnect(light, &QQuick3DParticleSpriteParticle::destroyed, self, &QQuick3DParticleSpriteParticle::onLightDestroyed);
     }
     self->m_lights.clear();
     self->updateFeatureLevel();
