@@ -244,17 +244,18 @@ struct QSSGShaderKeyTextureChannel : public QSSGShaderKeyUnsigned<2>
     }
 };
 
-struct QSSGShaderKeyImageMap : public QSSGShaderKeyUnsigned<5>
+struct QSSGShaderKeyImageMap : public QSSGShaderKeyUnsigned<6>
 {
     enum ImageMapBits {
         Enabled = 1 << 0,
         EnvMap = 1 << 1,
         LightProbe = 1 << 2,
         Identity = 1 << 3,
-        UsesUV1 = 1 << 4
+        UsesUV1 = 1 << 4,
+        Linear = 1 << 5
     };
 
-    explicit QSSGShaderKeyImageMap(const char *inName = "") : QSSGShaderKeyUnsigned<5>(inName) {}
+    explicit QSSGShaderKeyImageMap(const char *inName = "") : QSSGShaderKeyUnsigned<6>(inName) {}
 
     bool getBitValue(ImageMapBits imageBit, QSSGDataView<quint32> inKeySet) const
     {
@@ -289,6 +290,9 @@ struct QSSGShaderKeyImageMap : public QSSGShaderKeyUnsigned<5>
     bool isUsingUV1(QSSGDataView<quint32> inKeySet) const { return getBitValue(UsesUV1, inKeySet); }
     void setUsesUV1(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(UsesUV1, val, inKeySet); }
 
+    bool isLinear(QSSGDataView<quint32> inKeySet) const { return getBitValue(Linear, inKeySet); }
+    void setLinear(QSSGDataRef<quint32> inKeySet, bool val) { setBitValue(Linear, val, inKeySet); }
+
     void toString(QByteArray &ioStr, QSSGDataView<quint32> inKeySet) const
     {
         ioStr.append(name);
@@ -302,6 +306,8 @@ struct QSSGShaderKeyImageMap : public QSSGShaderKeyUnsigned<5>
         internalToString(ioStr, QByteArrayView("identity"), isIdentityTransform(inKeySet));
         ioStr.append(';');
         internalToString(ioStr, QByteArrayView("usesUV1"), isUsingUV1(inKeySet));
+        ioStr.append(';');
+        internalToString(ioStr, QByteArrayView("linear"), isLinear(inKeySet));
         ioStr.append('}');
     }
 };
@@ -500,15 +506,12 @@ struct QSSGShaderDefaultMaterialKeyProperties
         LightCount = QSSG_MAX_NUM_LIGHTS,
     };
     enum {
-        SingleChannelImageCount = 10,
+        SingleChannelImageCount = 13,
     };
     enum ImageMapNames {
         DiffuseMap = 0,
-        EmissiveMap,
-        SpecularMap,
-        BaseColorMap,
         BumpMap,
-        SpecularAmountMap,
+        SpecularMap,
         NormalMap,
         ClearcoatNormalMap,
         // single channel images
@@ -522,6 +525,9 @@ struct QSSGShaderDefaultMaterialKeyProperties
         ClearcoatRoughnessMap,
         TransmissionMap,
         ThicknessMap,
+        BaseColorMap,
+        SpecularAmountMap,
+        EmissiveMap,
 
         ImageMapCount,
         SingleChannelImagesFirst = OpacityMap
@@ -536,10 +542,10 @@ struct QSSGShaderDefaultMaterialKeyProperties
         ClearcoatChannel,
         ClearcoatRoughnessChannel,
         TransmissionChannel,
-        ThicknessChannel
-    };
-    enum {
-        MorphTargetCount = 8,
+        ThicknessChannel,
+        BaseColorChannel,
+        SpecularAmountChannel,
+        EmissiveChannel,
     };
 
     QSSGShaderKeyBoolean m_hasLighting;
@@ -549,9 +555,22 @@ struct QSSGShaderDefaultMaterialKeyProperties
     QSSGShaderKeyBoolean m_lightSpotFlags[LightCount];
     QSSGShaderKeyBoolean m_lightAreaFlags[LightCount];
     QSSGShaderKeyBoolean m_lightShadowFlags[LightCount];
+    QSSGShaderKeyUnsigned<16> m_lightShadowMapSize[LightCount];
+    QSSGShaderKeyUnsigned<4> m_lightSoftShadowQuality[LightCount];
     QSSGShaderKeyBoolean m_specularEnabled;
+    QSSGShaderKeyBoolean m_fresnelScaleBiasEnabled;
+    QSSGShaderKeyBoolean m_clearcoatFresnelScaleBiasEnabled;
     QSSGShaderKeyBoolean m_fresnelEnabled;
+    QSSGShaderKeyBoolean m_baseColorSingleChannelEnabled;
+    QSSGShaderKeyBoolean m_specularSingleChannelEnabled;
+    QSSGShaderKeyBoolean m_emissiveSingleChannelEnabled;
+    QSSGShaderKeyBoolean m_invertOpacityMapValue;
     QSSGShaderKeyBoolean m_vertexColorsEnabled;
+    QSSGShaderKeyBoolean m_vertexColorsMaskEnabled;
+    QSSGShaderKeyUnsigned<16> m_vertexColorRedMask;
+    QSSGShaderKeyUnsigned<16> m_vertexColorGreenMask;
+    QSSGShaderKeyUnsigned<16> m_vertexColorBlueMask;
+    QSSGShaderKeyUnsigned<16> m_vertexColorAlphaMask;
     QSSGShaderKeySpecularModel m_specularModel;
     QSSGShaderKeyImageMap m_imageMaps[ImageMapCount];
     QSSGShaderKeyTextureChannel m_textureChannels[SingleChannelImageCount];
@@ -567,14 +586,25 @@ struct QSSGShaderDefaultMaterialKeyProperties
     QSSGShaderKeyBoolean m_usesFloatJointIndices;
     qsizetype m_stringBufferSizeHint = 0;
     QSSGShaderKeyBoolean m_usesInstancing;
-    QSSGShaderKeyUnsigned<4> m_morphTargetCount;
-    QSSGShaderKeyVertexAttribute m_morphTargetAttributes[MorphTargetCount];
+    QSSGShaderKeyUnsigned<8> m_targetCount;
+    QSSGShaderKeyUnsigned<8> m_targetPositionOffset;
+    QSSGShaderKeyUnsigned<8> m_targetNormalOffset;
+    QSSGShaderKeyUnsigned<8> m_targetTangentOffset;
+    QSSGShaderKeyUnsigned<8> m_targetBinormalOffset;
+    QSSGShaderKeyUnsigned<8> m_targetTexCoord0Offset;
+    QSSGShaderKeyUnsigned<8> m_targetTexCoord1Offset;
+    QSSGShaderKeyUnsigned<8> m_targetColorOffset;
     QSSGShaderKeyBoolean m_blendParticles;
     QSSGShaderKeyBoolean m_clearcoatEnabled;
     QSSGShaderKeyBoolean m_transmissionEnabled;
     QSSGShaderKeyBoolean m_specularAAEnabled;
     QSSGShaderKeyBoolean m_lightmapEnabled;
     QSSGShaderKeyBoolean m_specularGlossyEnabled;
+    QSSGShaderKeyUnsigned<4> m_debugMode;
+    QSSGShaderKeyBoolean m_fogEnabled;
+    QSSGShaderKeyUnsigned<3> m_viewCount;
+    QSSGShaderKeyBoolean m_usesViewIndex;
+    QSSGShaderKeyUnsigned<3> m_orderIndependentTransparency;
 
     QSSGShaderDefaultMaterialKeyProperties()
         : m_hasLighting("hasLighting")
@@ -582,7 +612,16 @@ struct QSSGShaderDefaultMaterialKeyProperties
         , m_lightCount("lightCount")
         , m_specularEnabled("specularEnabled")
         , m_fresnelEnabled("fresnelEnabled")
+        , m_baseColorSingleChannelEnabled("baseColorSingleChannelEnabled")
+        , m_specularSingleChannelEnabled("specularSingleChannelEnabled")
+        , m_emissiveSingleChannelEnabled("emissiveSingleChannelEnabled")
+        , m_invertOpacityMapValue("invertOpacityMapValue")
         , m_vertexColorsEnabled("vertexColorsEnabled")
+        , m_vertexColorsMaskEnabled("vertexColorsMaskEnabled")
+        , m_vertexColorRedMask("vertexColorRedMask")
+        , m_vertexColorGreenMask("vertexColorGreenMask")
+        , m_vertexColorBlueMask("vertexColorBlueMask")
+        , m_vertexColorAlphaMask("vertexColorAlphaMask")
         , m_specularModel("specularModel")
         , m_boneCount("boneCount")
         , m_isDoubleSided("isDoubleSided")
@@ -595,13 +634,25 @@ struct QSSGShaderDefaultMaterialKeyProperties
         , m_vertexAttributes("vertexAttributes")
         , m_usesFloatJointIndices("usesFloatJointIndices")
         , m_usesInstancing("usesInstancing")
-        , m_morphTargetCount("morphTargetCount")
+        , m_targetCount("targetCount")
+        , m_targetPositionOffset("targetPositionOffset")
+        , m_targetNormalOffset("targetNormalOffset")
+        , m_targetTangentOffset("targetTangentOffset")
+        , m_targetBinormalOffset("targetBinormalOffset")
+        , m_targetTexCoord0Offset("targetTexCoord0Offset")
+        , m_targetTexCoord1Offset("targetTexCoord1Offset")
+        , m_targetColorOffset("targetColorOffset")
         , m_blendParticles("blendParticles")
         , m_clearcoatEnabled("clearcoatEnabled")
         , m_transmissionEnabled("transmissionEnabled")
         , m_specularAAEnabled("specularAAEnabled")
         , m_lightmapEnabled("lightmapEnabled")
         , m_specularGlossyEnabled("specularGlossyEnabled")
+        , m_debugMode("debugMode")
+        , m_fogEnabled("fogEnabled")
+        , m_viewCount("viewCount")
+        , m_usesViewIndex("usesViewIndex")
+        , m_orderIndependentTransparency("orderIndependentTransparency")
     {
         m_lightFlags[0].name = "light0HasPosition";
         m_lightFlags[1].name = "light1HasPosition";
@@ -667,6 +718,38 @@ struct QSSGShaderDefaultMaterialKeyProperties
         m_lightShadowFlags[13].name = "light13HasShadow";
         m_lightShadowFlags[14].name = "light14HasShadow";
 
+        m_lightShadowMapSize[0].name = "light0ShadowMapSize";
+        m_lightShadowMapSize[1].name = "light1ShadowMapSize";
+        m_lightShadowMapSize[2].name = "light2ShadowMapSize";
+        m_lightShadowMapSize[3].name = "light3ShadowMapSize";
+        m_lightShadowMapSize[4].name = "light4ShadowMapSize";
+        m_lightShadowMapSize[5].name = "light5ShadowMapSize";
+        m_lightShadowMapSize[6].name = "light6ShadowMapSize";
+        m_lightShadowMapSize[7].name = "light7ShadowMapSize";
+        m_lightShadowMapSize[8].name = "light8ShadowMapSize";
+        m_lightShadowMapSize[9].name = "light9ShadowMapSize";
+        m_lightShadowMapSize[10].name = "light10ShadowMapSize";
+        m_lightShadowMapSize[11].name = "light11ShadowMapSize";
+        m_lightShadowMapSize[12].name = "light12ShadowMapSize";
+        m_lightShadowMapSize[13].name = "light13ShadowMapSize";
+        m_lightShadowMapSize[14].name = "light14ShadowMapSize";
+
+        m_lightSoftShadowQuality[0].name = "light0SoftShadowQuality";
+        m_lightSoftShadowQuality[1].name = "light1SoftShadowQuality";
+        m_lightSoftShadowQuality[2].name = "light2SoftShadowQuality";
+        m_lightSoftShadowQuality[3].name = "light3SoftShadowQuality";
+        m_lightSoftShadowQuality[4].name = "light4SoftShadowQuality";
+        m_lightSoftShadowQuality[5].name = "light5SoftShadowQuality";
+        m_lightSoftShadowQuality[6].name = "light6SoftShadowQuality";
+        m_lightSoftShadowQuality[7].name = "light7SoftShadowQuality";
+        m_lightSoftShadowQuality[8].name = "light8SoftShadowQuality";
+        m_lightSoftShadowQuality[9].name = "light9SoftShadowQuality";
+        m_lightSoftShadowQuality[10].name = "light10SoftShadowQuality";
+        m_lightSoftShadowQuality[11].name = "light11SoftShadowQuality";
+        m_lightSoftShadowQuality[12].name = "light12SoftShadowQuality";
+        m_lightSoftShadowQuality[13].name = "light13SoftShadowQuality";
+        m_lightSoftShadowQuality[14].name = "light14SoftShadowQuality";
+
         m_imageMaps[0].name = "diffuseMap";
         m_imageMaps[1].name = "emissiveMap";
         m_imageMaps[2].name = "specularMap";
@@ -696,15 +779,8 @@ struct QSSGShaderDefaultMaterialKeyProperties
         m_textureChannels[7].name = "clearcoatRoughnessMap_channel";
         m_textureChannels[8].name = "transmissionMap_channel";
         m_textureChannels[9].name = "thicknessMap_channel";
-
-        m_morphTargetAttributes[0].name = "morphTarget0Attributes";
-        m_morphTargetAttributes[1].name = "morphTarget1Attributes";
-        m_morphTargetAttributes[2].name = "morphTarget2Attributes";
-        m_morphTargetAttributes[3].name = "morphTarget3Attributes";
-        m_morphTargetAttributes[4].name = "morphTarget4Attributes";
-        m_morphTargetAttributes[5].name = "morphTarget5Attributes";
-        m_morphTargetAttributes[6].name = "morphTarget6Attributes";
-        m_morphTargetAttributes[7].name = "morphTarget7Attributes";
+        m_textureChannels[10].name = "baseColorMap_channel";
+        m_textureChannels[11].name = "specularAmountMap_channel";
 
         init();
     }
@@ -728,9 +804,26 @@ struct QSSGShaderDefaultMaterialKeyProperties
         for (auto &lightShadowFlag : m_lightShadowFlags)
             inVisitor.visit(lightShadowFlag);
 
+        for (auto &lightShadowMapSize : m_lightShadowMapSize)
+            inVisitor.visit(lightShadowMapSize);
+
+        for (auto &softShadowQuality : m_lightSoftShadowQuality)
+            inVisitor.visit(softShadowQuality);
+
         inVisitor.visit(m_specularEnabled);
         inVisitor.visit(m_fresnelEnabled);
+        inVisitor.visit(m_fresnelScaleBiasEnabled);
+        inVisitor.visit(m_clearcoatFresnelScaleBiasEnabled);
+        inVisitor.visit(m_baseColorSingleChannelEnabled);
+        inVisitor.visit(m_specularSingleChannelEnabled);
+        inVisitor.visit(m_emissiveSingleChannelEnabled);
+        inVisitor.visit(m_invertOpacityMapValue);
         inVisitor.visit(m_vertexColorsEnabled);
+        inVisitor.visit(m_vertexColorsMaskEnabled);
+        inVisitor.visit(m_vertexColorRedMask);
+        inVisitor.visit(m_vertexColorGreenMask);
+        inVisitor.visit(m_vertexColorBlueMask);
+        inVisitor.visit(m_vertexColorAlphaMask);
         inVisitor.visit(m_specularModel);
 
         for (quint32 idx = 0, end = ImageMapCount; idx < end; ++idx)
@@ -750,15 +843,25 @@ struct QSSGShaderDefaultMaterialKeyProperties
         inVisitor.visit(m_vertexAttributes);
         inVisitor.visit(m_usesFloatJointIndices);
         inVisitor.visit(m_usesInstancing);
-        inVisitor.visit(m_morphTargetCount);
-        for (quint32 idx = 0, end = MorphTargetCount; idx < end; ++idx)
-            inVisitor.visit(m_morphTargetAttributes[idx]);
+        inVisitor.visit(m_targetCount);
+        inVisitor.visit(m_targetPositionOffset);
+        inVisitor.visit(m_targetNormalOffset);
+        inVisitor.visit(m_targetTangentOffset);
+        inVisitor.visit(m_targetBinormalOffset);
+        inVisitor.visit(m_targetTexCoord0Offset);
+        inVisitor.visit(m_targetTexCoord1Offset);
+        inVisitor.visit(m_targetColorOffset);
         inVisitor.visit(m_blendParticles);
         inVisitor.visit(m_clearcoatEnabled);
         inVisitor.visit(m_transmissionEnabled);
         inVisitor.visit(m_specularAAEnabled);
         inVisitor.visit(m_lightmapEnabled);
         inVisitor.visit(m_specularGlossyEnabled);
+        inVisitor.visit(m_debugMode);
+        inVisitor.visit(m_fogEnabled);
+        inVisitor.visit(m_viewCount);
+        inVisitor.visit(m_usesViewIndex);
+        inVisitor.visit(m_orderIndependentTransparency);
     }
 
     struct OffsetVisitor
@@ -811,7 +914,7 @@ struct QSSGShaderDefaultMaterialKeyProperties
         visitProperties(visitor);
 
         // If this assert fires, then the default material key needs more bits.
-        Q_ASSERT(visitor.offsetVisitor.m_offset < 416);
+        Q_ASSERT(visitor.offsetVisitor.m_offset < 768);
         // This is so we can do some guestimate of how big the string buffer needs
         // to be to avoid doing a lot of allocations when concatenating the strings.
         m_stringBufferSizeHint = visitor.stringSizeVisitor.size;
@@ -821,9 +924,9 @@ struct QSSGShaderDefaultMaterialKeyProperties
 struct QSSGShaderDefaultMaterialKey
 {
     enum {
-        DataBufferSize = 13,
+        DataBufferSize = 24,
     };
-    quint32 m_dataBuffer[DataBufferSize]; // 13 * 4 * 8 = 416 bits
+    quint32 m_dataBuffer[DataBufferSize]; // 24 * 4 * 8 = 768 bits
     size_t m_featureSetHash;
 
     explicit QSSGShaderDefaultMaterialKey(size_t inFeatureSetHash) : m_featureSetHash(inFeatureSetHash)
