@@ -378,6 +378,10 @@ QRhiTexture::Format QSSGBufferManager::toRhiFormat(const QSSGRenderTextureFormat
         return QRhiTexture::R32F;
     case QSSGRenderTextureFormat::RGBE8:
         return QRhiTexture::RGBA8;
+    case QSSGRenderTextureFormat::R32UI:
+        return QRhiTexture::R32UI;
+    case QSSGRenderTextureFormat::RGBA32UI:
+        return QRhiTexture::RGBA32UI;
     case QSSGRenderTextureFormat::RGB_DXT1:
         return QRhiTexture::BC1;
     case QSSGRenderTextureFormat::RGBA_DXT3:
@@ -387,32 +391,46 @@ QRhiTexture::Format QSSGBufferManager::toRhiFormat(const QSSGRenderTextureFormat
     case QSSGRenderTextureFormat::RGBA8_ETC2_EAC:
         return QRhiTexture::ETC2_RGBA8;
     case QSSGRenderTextureFormat::RGBA_ASTC_4x4:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_4x4:
         return QRhiTexture::ASTC_4x4;
     case QSSGRenderTextureFormat::RGBA_ASTC_5x4:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_5x4:
         return QRhiTexture::ASTC_5x4;
     case QSSGRenderTextureFormat::RGBA_ASTC_5x5:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_5x5:
         return QRhiTexture::ASTC_5x5;
     case QSSGRenderTextureFormat::RGBA_ASTC_6x5:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_6x5:
         return QRhiTexture::ASTC_6x5;
     case QSSGRenderTextureFormat::RGBA_ASTC_6x6:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_6x6:
         return QRhiTexture::ASTC_6x6;
     case QSSGRenderTextureFormat::RGBA_ASTC_8x5:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_8x5:
         return QRhiTexture::ASTC_8x5;
     case QSSGRenderTextureFormat::RGBA_ASTC_8x6:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_8x6:
         return QRhiTexture::ASTC_8x6;
     case QSSGRenderTextureFormat::RGBA_ASTC_8x8:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_8x8:
         return QRhiTexture::ASTC_8x8;
     case QSSGRenderTextureFormat::RGBA_ASTC_10x5:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_10x5:
         return QRhiTexture::ASTC_10x5;
     case QSSGRenderTextureFormat::RGBA_ASTC_10x6:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_10x6:
         return QRhiTexture::ASTC_10x6;
     case QSSGRenderTextureFormat::RGBA_ASTC_10x8:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_10x8:
         return QRhiTexture::ASTC_10x8;
     case QSSGRenderTextureFormat::RGBA_ASTC_10x10:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_10x10:
         return QRhiTexture::ASTC_10x10;
     case QSSGRenderTextureFormat::RGBA_ASTC_12x10:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_12x10:
         return QRhiTexture::ASTC_12x10;
     case QSSGRenderTextureFormat::RGBA_ASTC_12x12:
+    case QSSGRenderTextureFormat::SRGB8_Alpha8_ASTC_12x12:
         return QRhiTexture::ASTC_12x12;
 
 
@@ -2005,11 +2023,16 @@ static inline quint64 textureMemorySize(QRhiTexture *texture)
         R16F,
         R32F,
         RGB10A2,
+        R8UI,
+        R32UI,
+        RG32UI,
+        RGBA32UI,
         D16,
         D24,
         D24S8,
-        D32F,*/
-    static const quint64 pixelSizes[] = {0, 4, 4, 1, 2, 2, 4, 1, 2, 4, 2, 4, 4, 2, 4, 4, 4};
+        D32F,
+        D32FS8*/
+    static const quint64 pixelSizes[] = {0, 4, 4, 1, 2, 2, 4, 1, 2, 4, 2, 4, 4, 1, 4, 8, 16, 2, 4, 4, 4, 8};
     /*
         BC1,
         BC2,
@@ -2022,7 +2045,7 @@ static inline quint64 textureMemorySize(QRhiTexture *texture)
         ETC2_RGB8A1,
         ETC2_RGBA8,*/
     static const quint64 blockSizes[] = {8, 16, 16, 8, 16, 16, 16, 8, 8, 16};
-    Q_STATIC_ASSERT_X(QRhiTexture::BC1 == 17 && QRhiTexture::ETC2_RGBA8 == 26,
+    Q_STATIC_ASSERT_X(QRhiTexture::BC1 == 22 && QRhiTexture::ETC2_RGBA8 == 31,
                       "QRhiTexture format constant value missmatch.");
     if (format < QRhiTexture::BC1)
         s *= pixelSizes[format];
