@@ -7,6 +7,10 @@
 #include <QtQuick3DUtils/private/qssgdataref_p.h>
 #include <QtQuick3DUtils/private/qssglightmapuvgenerator_p.h>
 
+#include "meshoptimizer.h"
+
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 namespace QSSGMesh {
@@ -30,6 +34,11 @@ static const size_t VERTEX_BUFFER_ENTRY_STRUCT_SIZE = 16;
 static const size_t SUBSET_STRUCT_SIZE_V3_V4 = 40;
 // subset list: count, offset, minXYZ, maxXYZ, nameOffset, nameLength, lightmapSizeWidth, lightmapSizeHeight
 static const size_t SUBSET_STRUCT_SIZE_V5 = 48;
+// subset list: count, offset, minXYZ, maxXYZ, nameOffset, nameLength, lightmapSizeWidth, lightmapSizeHeight, lodCount
+static const size_t SUBSET_STRUCT_SIZE_V6 = 52;
+
+//lod entry: count, offset, distance
+static const size_t LOD_STRUCT_SIZE = 12;
 
 MeshInternal::MultiMeshInfo MeshInternal::readFileHeader(QIODevice *device)
 {
@@ -95,22 +104,35 @@ quint64 MeshInternal::readMeshData(QIODevice *device, quint64 offset, Mesh *mesh
     inputStream >> header->fileId >> header->fileVersion >> header->flags >> header->sizeInBytes;
     if (!header->isValid()) {
         qWarning() << "Mesh data invalid";
+        if (header->fileId == MeshDataHeader::FILE_ID) {
+            if (header->fileVersion > MeshDataHeader::FILE_VERSION)
+                qWarning() << "File version " << header->fileVersion << " newer than " << MeshDataHeader::FILE_VERSION;
+            if (header->fileVersion < MeshDataHeader::LEGACY_MESH_FILE_VERSION)
+                qWarning() << "File version " << header->fileVersion << " older than " << MeshDataHeader::LEGACY_MESH_FILE_VERSION;
+        } else {
+            qWarning() << "Invalid file ID" << header->fileId;
+        }
         return 0;
     }
 
     MeshInternal::MeshOffsetTracker offsetTracker(offset + MESH_HEADER_STRUCT_SIZE);
     Q_ASSERT(offsetTracker.offset() == device->pos());
 
-    quint32 vertexBufferEntriesOffset; // unused
+    quint32 targetBufferEntriesCount;
     quint32 vertexBufferEntriesCount;
-    quint32 vertexBufferDataOffset; // unused
+    quint32 targetBufferDataSize;
     quint32 vertexBufferDataSize;
-    inputStream >> vertexBufferEntriesOffset
+    inputStream >> targetBufferEntriesCount
                 >> vertexBufferEntriesCount
                 >> mesh->m_vertexBuffer.stride
-                >> vertexBufferDataOffset
+                >> targetBufferDataSize
                 >> vertexBufferDataSize;
 
+    if (!header->hasSeparateTargetBuffer()) {
+        targetBufferEntriesCount = 0;
+        targetBufferDataSize = 0;
+    }
+
     quint32 indexBufferComponentType;
     quint32 indexBufferDataOffset;
     quint32 indexBufferDataSize;
@@ -119,14 +141,14 @@ quint64 MeshInternal::readMeshData(QIODevice *device, quint64 offset, Mesh *mesh
                 >> indexBufferDataSize;
     mesh->m_indexBuffer.componentType = Mesh::ComponentType(indexBufferComponentType);
 
-    quint32 subsetsOffsets; // unused
+    quint32 targetCount;
     quint32 subsetsCount;
-    inputStream >> subsetsOffsets >> subsetsCount;
+    inputStream >> targetCount >> subsetsCount;
+    mesh->m_targetBuffer.numTargets = targetCount;
 
     quint32 jointsOffsets; // unused
     quint32 jointsCount; // unused
     inputStream >> jointsOffsets >> jointsCount;
-
     quint32 drawMode;
     quint32 winding;
     inputStream >> drawMode >> winding;
@@ -153,6 +175,10 @@ quint64 MeshInternal::readMeshData(QIODevice *device, quint64 offset, Mesh *mesh
         device->read(alignPadding, alignAmount);
 
     // vertex buffer entry names
+    quint32 numTargets = 0;
+    // used for recording the target attributes supported by the mesh
+    // and re-construting it when meeting attr_unsupported
+    QList<QByteArray> attrNames;
     for (auto &entry : mesh->m_vertexBuffer.entries) {
         quint32 nameLength;
         inputStream >> nameLength;
@@ -162,6 +188,52 @@ quint64 MeshInternal::readMeshData(QIODevice *device, quint64 offset, Mesh *mesh
         alignAmount = offsetTracker.alignedAdvance(nameLength);
         if (alignAmount)
             device->read(alignPadding, alignAmount);
+        // Old morph meshes' target attributes were appended sequentially
+        // behind vertex attributes. However, since the number of targets are restricted by 8
+        // the other attributes were named by "attr_unsupported"
+        // So just checking numTargets is safe with the above assumption and
+        // it will try to reconstruct the unsupported attributes.
+        if (numTargets > 0 || (!header->hasSeparateTargetBuffer() && entry.name.startsWith("attr_t"))) {
+            if (entry.name.sliced(6).startsWith("pos")) {
+                const quint32 targetId = entry.name.mid(9).toUInt();
+                // All the attributes of the first target should be recorded correctly.
+                if (targetId == 0)
+                    attrNames.append(MeshInternal::getPositionAttrName());
+                numTargets = qMax(numTargets, targetId + 1);
+                entry.name = MeshInternal::getPositionAttrName();
+                mesh->m_targetBuffer.entries.append(entry);
+                targetBufferEntriesCount++;
+            } else if (entry.name.sliced(6).startsWith("norm")) {
+                const quint32 targetId = entry.name.mid(10).toUInt();
+                if (targetId == 0)
+                    attrNames.append(MeshInternal::getNormalAttrName());
+                numTargets = qMax(numTargets, targetId + 1);
+                entry.name = MeshInternal::getNormalAttrName();
+                mesh->m_targetBuffer.entries.append(entry);
+                targetBufferEntriesCount++;
+            } else if (entry.name.sliced(6).startsWith("tan")) {
+                const quint32 targetId = entry.name.mid(9).toUInt();
+                if (targetId == 0)
+                    attrNames.append(MeshInternal::getTexTanAttrName());
+                numTargets = qMax(numTargets, targetId + 1);
+                entry.name = MeshInternal::getTexTanAttrName();
+                mesh->m_targetBuffer.entries.append(entry);
+                targetBufferEntriesCount++;
+            } else if (entry.name.sliced(6).startsWith("binorm")) {
+                const quint32 targetId = entry.name.mid(12).toUInt();
+                if (targetId == 0)
+                    attrNames.append(MeshInternal::getTexBinormalAttrName());
+                numTargets = qMax(numTargets, targetId + 1);
+                entry.name = MeshInternal::getTexBinormalAttrName();
+                mesh->m_targetBuffer.entries.append(entry);
+                targetBufferEntriesCount++;
+            } else if (entry.name.startsWith("attr_unsupported")) {
+                // Reconstruct
+                entry.name = attrNames[targetBufferEntriesCount % attrNames.size()];
+                mesh->m_targetBuffer.entries.append(entry);
+                targetBufferEntriesCount++;
+            }
+        }
     }
 
     mesh->m_vertexBuffer.data = device->read(vertexBufferDataSize);
@@ -202,7 +274,14 @@ quint64 MeshInternal::readMeshData(QIODevice *device, quint64 offset, Mesh *mesh
             quint32 height = 0;
             inputStream >> width >> height;
             subset.lightmapSizeHint = QSize(width, height);
-            subsetByteSize += SUBSET_STRUCT_SIZE_V5;
+            if (header->hasLodDataHint()) {
+                quint32 lodCount = 0;
+                inputStream >> lodCount;
+                subset.lodCount = lodCount;
+                subsetByteSize += SUBSET_STRUCT_SIZE_V6;
+            } else {
+                subsetByteSize += SUBSET_STRUCT_SIZE_V5;
+            }
         } else {
             subset.lightmapSizeHint = QSize(0, 0);
             subsetByteSize += SUBSET_STRUCT_SIZE_V3_V4;
@@ -221,8 +300,88 @@ quint64 MeshInternal::readMeshData(QIODevice *device, quint64 offset, Mesh *mesh
             device->read(alignPadding, alignAmount);
     }
 
-    for (const MeshInternal::Subset &internalSubset : internalSubsets)
-        mesh->m_subsets.append(internalSubset.toMeshSubset());
+    quint32 lodByteSize = 0;
+    for (const MeshInternal::Subset &internalSubset : internalSubsets) {
+        auto meshSubset = internalSubset.toMeshSubset();
+        // Read Level of Detail data here
+        for (auto &lod : meshSubset.lods) {
+            quint32 count = 0;
+            quint32 offset = 0;
+            float distance = 0.0;
+            inputStream >> count >> offset >> distance;
+            lod.count = count;
+            lod.offset = offset;
+            lod.distance = distance;
+            lodByteSize += LOD_STRUCT_SIZE;
+        }
+
+        mesh->m_subsets.append(meshSubset);
+    }
+    alignAmount = offsetTracker.alignedAdvance(lodByteSize);
+    if (alignAmount)
+        device->read(alignPadding, alignAmount);
+
+
+    // Data for morphTargets
+    if (targetBufferEntriesCount > 0) {
+        if (header->hasSeparateTargetBuffer()) {
+            entriesByteSize = 0;
+            for (quint32 i = 0; i < targetBufferEntriesCount; ++i) {
+                Mesh::VertexBufferEntry targetBufferEntry;
+                quint32 componentType;
+                quint32 nameOffset; // unused
+                inputStream >> nameOffset
+                            >> componentType
+                            >> targetBufferEntry.componentCount
+                            >> targetBufferEntry.offset;
+                targetBufferEntry.componentType = Mesh::ComponentType(componentType);
+                mesh->m_targetBuffer.entries.append(targetBufferEntry);
+                entriesByteSize += VERTEX_BUFFER_ENTRY_STRUCT_SIZE;
+            }
+            alignAmount = offsetTracker.alignedAdvance(entriesByteSize);
+            if (alignAmount)
+                device->read(alignPadding, alignAmount);
+
+            for (auto &entry : mesh->m_targetBuffer.entries) {
+                quint32 nameLength;
+                inputStream >> nameLength;
+                offsetTracker.advance(sizeof(quint32));
+                const QByteArray nameWithZeroTerminator = device->read(nameLength);
+                entry.name = QByteArray(nameWithZeroTerminator.constData(), qMax(0, nameWithZeroTerminator.size() - 1));
+                alignAmount = offsetTracker.alignedAdvance(nameLength);
+                if (alignAmount)
+                    device->read(alignPadding, alignAmount);
+            }
+
+            mesh->m_targetBuffer.data = device->read(targetBufferDataSize);
+        } else {
+            // remove target entries from vertexbuffer entries
+            mesh->m_vertexBuffer.entries.remove(vertexBufferEntriesCount - targetBufferEntriesCount,
+                                                targetBufferEntriesCount);
+            const quint32 vertexCount = vertexBufferDataSize / mesh->m_vertexBuffer.stride;
+            const quint32 targetEntryTexWidth = qCeil(qSqrt(vertexCount));
+            const quint32 targetCompStride = targetEntryTexWidth * targetEntryTexWidth * 4 * sizeof(float);
+            mesh->m_targetBuffer.data.resize(targetCompStride * targetBufferEntriesCount);
+            const quint32 numComps = targetBufferEntriesCount / numTargets;
+            for (quint32 i = 0; i < targetBufferEntriesCount; ++i) {
+                auto &entry = mesh->m_targetBuffer.entries[i];
+                char *dstBuf = mesh->m_targetBuffer.data.data()
+                                + (i / numComps) * targetCompStride
+                                + (i % numComps) * (targetCompStride * numTargets);
+                const char *srcBuf = mesh->m_vertexBuffer.data.constData() + entry.offset;
+                for (quint32 j = 0; j < vertexCount; ++j) {
+                    // The number of old target components is fixed as 3
+                    memcpy(dstBuf + j * 4 * sizeof(float),
+                           srcBuf + j * mesh->m_vertexBuffer.stride,
+                           3 * sizeof(float));
+                }
+                entry.offset = i * targetCompStride;
+            }
+            // now we don't need to have redundant targetbuffer entries
+            mesh->m_targetBuffer.entries.remove(numComps, targetBufferEntriesCount - numComps);
+            mesh->m_targetBuffer.numTargets = numTargets;
+        }
+    }
 
     return header->sizeInBytes;
 }
@@ -260,10 +419,12 @@ quint64 MeshInternal::writeMeshData(QIODevice *device, const Mesh &mesh)
     const quint32 vertexBufferEntriesCount = mesh.m_vertexBuffer.entries.size();
     const quint32 vertexBufferDataSize = mesh.m_vertexBuffer.data.size();
     const quint32 vertexBufferStride = mesh.m_vertexBuffer.stride;
-    outputStream << quint32(0) // legacy offset
+    const quint32 targetBufferEntriesCount = mesh.m_targetBuffer.entries.count();
+    const quint32 targetBufferDataSize = mesh.m_targetBuffer.data.size();
+    outputStream << targetBufferEntriesCount
                  << vertexBufferEntriesCount
                  << vertexBufferStride;
-    outputStream << quint32(0) // legacy offset
+    outputStream << targetBufferDataSize
                  << vertexBufferDataSize;
 
     const quint32 indexBufferDataSize = mesh.m_indexBuffer.data.size();
@@ -272,8 +433,9 @@ quint64 MeshInternal::writeMeshData(QIODevice *device, const Mesh &mesh)
     outputStream << quint32(0) // legacy offset
                  << indexBufferDataSize;
 
+    const quint32 targetCount = mesh.m_targetBuffer.numTargets;
     const quint32 subsetsCount = mesh.m_subsets.size();
-    outputStream << quint32(0) // legacy offset
+    outputStream << targetCount
                  << subsetsCount;
 
     outputStream << quint32(0) // legacy offset
@@ -336,6 +498,7 @@ quint64 MeshInternal::writeMeshData(QIODevice *device, const Mesh &mesh)
         const quint32 nameLength = subset.name.size() + 1;
         const quint32 lightmapSizeHintWidth = qMax(0, subset.lightmapSizeHint.width());
         const quint32 lightmapSizeHintHeight = qMax(0, subset.lightmapSizeHint.height());
+        const quint32 lodCount = subset.lods.size();
         outputStream << subsetCount
                      << subsetOffset
                      << minX
@@ -348,7 +511,8 @@ quint64 MeshInternal::writeMeshData(QIODevice *device, const Mesh &mesh)
                      << nameLength;
         outputStream << lightmapSizeHintWidth
                      << lightmapSizeHintHeight;
-        subsetByteSize += SUBSET_STRUCT_SIZE_V5;
+        outputStream << lodCount;
+        subsetByteSize += SUBSET_STRUCT_SIZE_V6;
     }
     alignAmount = offsetTracker.alignedAdvance(subsetByteSize);
     if (alignAmount)
@@ -364,6 +528,50 @@ quint64 MeshInternal::writeMeshData(QIODevice *device, const Mesh &mesh)
             device->write(alignPadding, alignAmount);
     }
 
+    // LOD data
+    quint32 lodDataByteSize = 0;
+    for (quint32 i = 0; i < subsetsCount; ++i) {
+        const Mesh::Subset &subset(mesh.m_subsets[i]);
+        for (auto lod : subset.lods) {
+            const quint32 count = lod.count;
+            const quint32 offset = lod.offset;
+            const float distance = lod.distance;
+            outputStream << count << offset << distance;
+            lodDataByteSize += LOD_STRUCT_SIZE;
+        }
+    }
+    alignAmount = offsetTracker.alignedAdvance(lodDataByteSize);
+    if (alignAmount)
+        device->write(alignPadding, alignAmount);
+
+    // Data for morphTargets
+    for (quint32 i = 0; i < targetBufferEntriesCount; ++i) {
+        const Mesh::VertexBufferEntry &entry(mesh.m_targetBuffer.entries[i]);
+        const quint32 componentType = quint32(entry.componentType);
+        const quint32 componentCount = entry.componentCount;
+        const quint32 offset = entry.offset;
+        outputStream << quint32(0) // legacy offset
+                     << componentType
+                     << componentCount
+                     << offset;
+        entriesByteSize += VERTEX_BUFFER_ENTRY_STRUCT_SIZE;
+    }
+    alignAmount = offsetTracker.alignedAdvance(entriesByteSize);
+    if (alignAmount)
+        device->write(alignPadding, alignAmount);
+
+    for (quint32 i = 0; i < targetBufferEntriesCount; ++i) {
+        const Mesh::VertexBufferEntry &entry(mesh.m_targetBuffer.entries[i]);
+        const quint32 nameLength = entry.name.size() + 1;
+        outputStream << nameLength;
+        device->write(entry.name.constData(), nameLength); // with zero terminator included
+        alignAmount = offsetTracker.alignedAdvance(sizeof(quint32) + nameLength);
+        if (alignAmount)
+            device->write(alignPadding, alignAmount);
+    }
+
+    device->write(mesh.m_targetBuffer.data.constData(), targetBufferDataSize);
+
     const quint32 endPos = device->pos();
     const quint32 sizeInBytes = endPos - startPos;
     device->seek(endPos);
@@ -417,7 +625,9 @@ static inline quint32 getAlignedOffset(quint32 offset, quint32 align)
 Mesh Mesh::fromAssetData(const QVector<AssetVertexEntry> &vbufEntries,
                          const QByteArray &indexBufferData,
                          ComponentType indexComponentType,
-                         const QVector<AssetMeshSubset> &subsets)
+                         const QVector<AssetMeshSubset> &subsets,
+                         quint32 numTargets,
+                         quint32 numTargetComps)
 {
     Mesh mesh;
     quint32 currentOffset = 0;
@@ -425,40 +635,68 @@ Mesh Mesh::fromAssetData(const QVector<AssetVertexEntry> &vbufEntries,
     quint32 numItems = 0;
     bool ok = true;
 
+    mesh.m_targetBuffer.numTargets = numTargets;
+    quint32 targetCurrentComp = 0;
+    quint32 targetCompStride = 0;
+
+    QVector<AssetVertexEntry> vEntries;
     for (const AssetVertexEntry &entry : vbufEntries) {
         // Ignore entries with no data.
         if (entry.data.isEmpty())
             continue;
 
-        const quint32 alignment = MeshInternal::byteSizeForComponentType(entry.componentType);
-        bufferAlignment = qMax(bufferAlignment, alignment);
-        const quint32 byteSize = alignment * entry.componentCount;
-
-        if (entry.data.size() % alignment != 0) {
-            Q_ASSERT(false);
-            ok = false;
-        }
-
-        quint32 localNumItems = entry.data.size() / byteSize;
-        if (numItems == 0) {
-            numItems = localNumItems;
-        } else if (numItems != localNumItems) {
-            Q_ASSERT(false);
-            ok = false;
-            numItems = qMin(numItems, localNumItems);
-        }
-
-        currentOffset = getAlignedOffset(currentOffset, alignment);
-
         VertexBufferEntry meshEntry;
         meshEntry.componentType = entry.componentType;
         meshEntry.componentCount = entry.componentCount;
-        meshEntry.offset = currentOffset;
         meshEntry.name = entry.name;
 
-        mesh.m_vertexBuffer.entries.append(meshEntry);
+        if (entry.morphTargetId < 0) {
+            const quint32 alignment = MeshInternal::byteSizeForComponentType(entry.componentType);
+            const quint32 byteSize = alignment * entry.componentCount;
 
-        currentOffset += byteSize;
+            if (entry.data.size() % alignment != 0) {
+                Q_ASSERT(false);
+                ok = false;
+            }
+
+            quint32 localNumItems = entry.data.size() / byteSize;
+            if (numItems == 0) {
+                numItems = localNumItems;
+            } else if (numItems != localNumItems) {
+                Q_ASSERT(false);
+                ok = false;
+                numItems = qMin(numItems, localNumItems);
+            }
+
+            currentOffset = getAlignedOffset(currentOffset, alignment);
+            meshEntry.offset = currentOffset;
+
+            mesh.m_vertexBuffer.entries.append(meshEntry);
+            currentOffset += byteSize;
+            bufferAlignment = qMax(bufferAlignment, alignment);
+            vEntries.append(entry);
+        } else {
+            if (!targetCompStride) {
+                const quint32 targetEntrySize = entry.data.size();
+                quint32 targetEntryTexWidth = qCeil(qSqrt(((targetEntrySize + 15) >> 4)));
+                targetCompStride = targetEntryTexWidth * targetEntryTexWidth * 4 * sizeof(float);
+                mesh.m_targetBuffer.data.resize(targetCompStride * numTargets * numTargetComps);
+            }
+
+            // At assets, these entries are appended sequentially from target 0 to target N - 1
+            // It is safe to calculate the offset by the data size
+            meshEntry.offset = (targetCurrentComp * numTargets + entry.morphTargetId)
+                                    * targetCompStride;
+            memcpy(mesh.m_targetBuffer.data.data() + meshEntry.offset,
+                   entry.data.constData(), entry.data.size());
+
+            // Note: the targetBuffer will not be interleaved,
+            // data will be just appended in order and used for a texture array.
+            if (entry.morphTargetId == 0)
+                mesh.m_targetBuffer.entries.append(meshEntry);
+
+            targetCurrentComp = (targetCurrentComp + 1 < numTargetComps) ? targetCurrentComp + 1 : 0;
+        }
     }
 
     if (!ok)
@@ -469,7 +707,7 @@ Mesh Mesh::fromAssetData(const QVector<AssetVertexEntry> &vbufEntries,
     // Packed interleave the data.
     for (quint32 idx = 0; idx < numItems; ++idx) {
         quint32 dataOffset = 0;
-        for (const AssetVertexEntry &entry : vbufEntries) {
+        for (const AssetVertexEntry &entry : vEntries) {
             if (entry.data.isEmpty())
                 continue;
 
@@ -477,7 +715,6 @@ Mesh Mesh::fromAssetData(const QVector<AssetVertexEntry> &vbufEntries,
             const quint32 byteSize = alignment * entry.componentCount;
             const quint32 offset = byteSize * idx;
             const quint32 newOffset = getAlignedOffset(dataOffset, alignment);
-
             if (newOffset != dataOffset) {
                 QByteArray filler(newOffset - dataOffset, '\0');
                 mesh.m_vertexBuffer.data.append(filler);
@@ -498,6 +735,7 @@ Mesh Mesh::fromAssetData(const QVector<AssetVertexEntry> &vbufEntries,
         meshSubset.count = subset.count;
         meshSubset.offset = subset.offset;
 
+        // TODO: QTBUG-102026
         if (subset.boundsPositionEntryIndex != std::numeric_limits<quint32>::max()) {
             const QSSGBounds3 bounds = MeshInternal::calculateSubsetBounds(
                     mesh.m_vertexBuffer.entries[subset.boundsPositionEntryIndex],
@@ -512,6 +750,7 @@ Mesh Mesh::fromAssetData(const QVector<AssetVertexEntry> &vbufEntries,
         }
 
         meshSubset.lightmapSizeHint = QSize(subset.lightmapWidth, subset.lightmapHeight);
+        meshSubset.lods = subset.lods;
 
         mesh.m_subsets.append(meshSubset);
     }
@@ -571,18 +810,6 @@ Mesh Mesh::fromRuntimeData(const RuntimeMeshData &data, QString *error)
             case RuntimeMeshData::Attribute::ColorSemantic:
                 name = MeshInternal::getColorAttrName();
                 break;
-            case RuntimeMeshData::Attribute::TargetPositionSemantic:
-                name = MeshInternal::getTargetPositionAttrName(0);
-                break;
-            case RuntimeMeshData::Attribute::TargetNormalSemantic:
-                name = MeshInternal::getTargetNormalAttrName(0);
-                break;
-            case RuntimeMeshData::Attribute::TargetTangentSemantic:
-                name = MeshInternal::getTargetTangentAttrName(0);
-                break;
-            case RuntimeMeshData::Attribute::TargetBinormalSemantic:
-                name = MeshInternal::getTargetBinormalAttrName(0);
-                break;
             default:
                 *error = QObject::tr("Warning: Invalid attribute semantic: %1")
                         .arg(att.semantic);
@@ -605,6 +832,81 @@ Mesh Mesh::fromRuntimeData(const RuntimeMeshData &data, QString *error)
     mesh.m_subsets = data.m_subsets;
     mesh.m_indexBuffer.data = data.m_indexBuffer;
 
+    if (!data.m_targetBuffer.isEmpty()) {
+        const quint32 vertexCount = data.m_vertexBuffer.size() / data.m_stride;
+        const quint32 targetEntryTexWidth = qCeil(qSqrt(vertexCount));
+        const quint32 targetCompStride = targetEntryTexWidth * targetEntryTexWidth * 4 * sizeof(float);
+        mesh.m_targetBuffer.data.resize(targetCompStride * data.m_targetAttributeCount);
+
+        QVarLengthArray<RuntimeMeshData::TargetAttribute> sortedAttribs(
+                                            data.m_targetAttributes,
+                                            data.m_targetAttributes + data.m_targetAttributeCount);
+        std::sort(sortedAttribs.begin(), sortedAttribs.end(),
+                  [] (RuntimeMeshData::TargetAttribute a, RuntimeMeshData::TargetAttribute b) {
+                  return (a.targetId == b.targetId) ? a.attr.semantic < b.attr.semantic :
+                                                      a.targetId < b.targetId; });
+        for (int i = 0; i < data.m_targetAttributeCount; ++i) {
+            const RuntimeMeshData::Attribute &att = sortedAttribs[i].attr;
+            const int stride = (sortedAttribs[i].stride < 1) ? att.componentCount() * sizeof(float)
+                                                             : sortedAttribs[i].stride;
+            const char *name = nullptr;
+            switch (att.semantic) {
+            case RuntimeMeshData::Attribute::PositionSemantic:
+                name = MeshInternal::getPositionAttrName();
+                break;
+            case RuntimeMeshData::Attribute::NormalSemantic:
+                name = MeshInternal::getNormalAttrName();
+                break;
+            case RuntimeMeshData::Attribute::TexCoord0Semantic:
+                name = MeshInternal::getUV0AttrName();
+                break;
+            case RuntimeMeshData::Attribute::TexCoord1Semantic:
+                name = MeshInternal::getUV1AttrName();
+                break;
+            case RuntimeMeshData::Attribute::TangentSemantic:
+                name = MeshInternal::getTexTanAttrName();
+                break;
+            case RuntimeMeshData::Attribute::BinormalSemantic:
+                name = MeshInternal::getTexBinormalAttrName();
+                break;
+            case RuntimeMeshData::Attribute::IndexSemantic:
+            case RuntimeMeshData::Attribute::JointSemantic:
+            case RuntimeMeshData::Attribute::WeightSemantic:
+                *error = QObject::tr("Warning: Invalid target attribute semantic: %1")
+                        .arg(att.semantic);
+                continue;
+            case RuntimeMeshData::Attribute::ColorSemantic:
+                name = MeshInternal::getColorAttrName();
+                break;
+            default:
+                *error = QObject::tr("Warning: Invalid target attribute semantic: %1")
+                        .arg(att.semantic);
+                return Mesh();
+            }
+            char *dstBuf = mesh.m_targetBuffer.data.data() + i * targetCompStride;
+            const char *srcBuf = data.m_targetBuffer.constData() + att.offset;
+            Q_ASSERT(att.componentType == Mesh::ComponentType::Float32);
+            if (stride == 4 * sizeof(float)) {
+                memcpy(dstBuf, srcBuf, vertexCount * stride);
+            } else {
+                for (quint32 j = 0; j < vertexCount; ++j) {
+                    memcpy(dstBuf + j * 4 * sizeof(float),
+                           srcBuf + j * stride,
+                           att.componentCount() * sizeof(float));
+                }
+            }
+
+            if (sortedAttribs[i].targetId == 0) {
+                VertexBufferEntry entry;
+                entry.componentType = att.componentType;
+                entry.componentCount = att.componentCount();
+                entry.offset = i * targetCompStride;
+                entry.name = name;
+                mesh.m_targetBuffer.entries.append(entry);
+            }
+        }
+        mesh.m_targetBuffer.numTargets = data.m_targetAttributeCount / mesh.m_targetBuffer.entries.size();
+    }
     return mesh;
 }
 
@@ -944,6 +1246,21 @@ bool Mesh::createLightmapUVChannel(uint lightmapBaseResolution)
     return true;
 }
 
+size_t simplifyMesh(unsigned int *destination, const unsigned int *indices, size_t indexCount, const float *vertexPositions, size_t vertexCount, size_t vertexPositionsStride, size_t targetIndexCount, float targetError, unsigned int options, float *resultError)
+{
+    return meshopt_simplify(destination, indices, indexCount, vertexPositions, vertexCount, vertexPositionsStride, targetIndexCount, targetError, options, resultError);
+}
+
+float simplifyScale(const float *vertexPositions, size_t vertexCount, size_t vertexPositionsStride)
+{
+    return meshopt_simplifyScale(vertexPositions, vertexCount, vertexPositionsStride);
+}
+
+void optimizeVertexCache(unsigned int *destination, const unsigned int *indices, size_t indexCount, size_t vertexCount)
+{
+    meshopt_optimizeVertexCache(destination, indices, indexCount, vertexCount);
+}
+
 } // namespace QSSGMesh
 
 QT_END_NAMESPACE
