@@ -20,6 +20,8 @@
 #include <QtQuick/qquickwindow.h>
 #include <QtQuick/qquickrendercontrol.h>
 
+#include <QtGui/qquaternion.h>
+
 #include <QtCore/qobject.h>
 
 #include <openxr/openxr_reflection.h>
@@ -49,6 +51,10 @@
 # endif // XR_USE_GRAPHICS_API_OPENGL_ES
 #endif // XR_USE_PLATFORM_ANDROID
 
+#ifdef XR_USE_GRAPHICS_API_METAL
+# include "qopenxrgraphics_metal_p.h"
+#endif // XR_USE_GRAPHICS_API_METAL
+
 static XrReferenceSpaceType getXrReferenceSpaceType(QtQuick3DXr::ReferenceSpace referenceSpace)
 {
     switch (referenceSpace) {
@@ -332,8 +338,6 @@ void QQuick3DXrManagerPrivate::doRenderFrame()
 
     QVector<XrCompositionLayerBaseHeader*> layers;
 
-    m_passthroughSupported = supportsPassthrough();
-
     XrCompositionLayerPassthroughFB passthroughCompLayer{};
     passthroughCompLayer.type = XR_TYPE_COMPOSITION_LAYER_PASSTHROUGH_FB;
     if (m_enablePassthrough && m_passthroughSupported) {
@@ -460,6 +464,10 @@ bool QQuick3DXrManagerPrivate::initialize()
     if (graphicsAPI == QSGRendererInterface::OpenGL)
         m_graphics = new QOpenXRGraphicsOpenGLES;
 #endif
+#ifdef XR_USE_GRAPHICS_API_METAL
+    if (graphicsAPI == QSGRendererInterface::Metal)
+        m_graphics = new QOpenXRGraphicsMetal;
+#endif
 
     if (!m_graphics) {
         qWarning() << "The Qt Quick Scenegraph is not using a supported RHI mode:" << graphicsAPI;
@@ -1137,11 +1145,12 @@ void QQuick3DXrManagerPrivate::setMultiViewRenderingEnabled(bool enable)
     }
 }
 
-void QQuick3DXrManagerPrivate::setPassthroughEnabled(bool enable)
+bool QQuick3DXrManagerPrivate::setPassthroughEnabled(bool enable)
 {
-    m_enablePassthrough = enable;
+    m_passthroughSupported = supportsPassthrough();
 
     if (m_passthroughSupported) {
+        m_enablePassthrough = enable;
         if (m_enablePassthrough) {
             if (m_passthroughFeature == XR_NULL_HANDLE)
                 createMetaQuestPassthrough(); // Create and start
@@ -1161,6 +1170,7 @@ void QQuick3DXrManagerPrivate::setPassthroughEnabled(bool enable)
                 pauseMetaQuestPassthrough();
         }
     }
+    return m_passthroughSupported;
 }
 
 void QQuick3DXrManagerPrivate::setDepthSubmissionEnabled(bool enable)
@@ -1984,8 +1994,7 @@ XrResult QQuick3DXrManagerPrivate::createXrInstance()
     // Passthrough extensions (require manifest feature to work)
     // <uses-feature android:name="com.oculus.feature.PASSTHROUGH" android:required="true" />
     uint32_t passthroughSpecVersion = 0;
-    m_passthroughSupported = isExtensionSupported(XR_FB_PASSTHROUGH_EXTENSION_NAME, extensionProperties, &passthroughSpecVersion);
-    if (m_passthroughSupported) {
+    if (isExtensionSupported(XR_FB_PASSTHROUGH_EXTENSION_NAME, extensionProperties, &passthroughSpecVersion)) {
         qCDebug(lcQuick3DXr, "Passthrough extension is supported, spec version %u", passthroughSpecVersion);
         enabledExtensions.append(XR_FB_PASSTHROUGH_EXTENSION_NAME);
     } else {
